@@ -479,10 +479,11 @@ INSTRTXT *create_instrument0(CSOUND *csound, TREE *root,
   OPTXT *op;
   TREE *current;
   MYFLT sr = FL(-1.0), kr = FL(-1.0), ksmps = FL(-1.0), nchnls = DFLT_NCHNLS,
-        inchnls = FL(0.0), _0dbfs = FL(-1.0);
+        inchnls = -FL(1.0), _0dbfs = FL(-1.0);
   int krdef = 0; //, ksmpsdef = 0, srdef = 0;
   double A4 = 0.0;
   CS_TYPE *rType = (CS_TYPE *)&CS_VAR_TYPE_R;
+  OPARMS *O = csound->oparms;
 
   addGlobalVariable(csound, engineState, rType, "sr", NULL);
   addGlobalVariable(csound, engineState, rType, "kr", NULL);
@@ -561,7 +562,7 @@ INSTRTXT *create_instrument0(CSOUND *csound, TREE *root,
           uval = (val <= 0 ? 1u : (unsigned int)val);
           nchnls = uval;
         } else if (current->left->type == NCHNLSI_TOKEN) {
-          uval = (val <= 0 ? 1u : (unsigned int)val);
+          uval = (val < 0 ? 1u : (unsigned int)val);
           inchnls = uval;
         } else if (current->left->type == ZERODBFS_TOKEN) {
           _0dbfs = val;
@@ -579,14 +580,14 @@ INSTRTXT *create_instrument0(CSOUND *csound, TREE *root,
   /* Deal with defaults and consistency */
   if (ksmps == FL(-1.0)) {
     if (sr == FL(-1.0))
-      sr = DFLT_SR;
+      sr = csound->oparms->sr_default;//   DFLT_SR;
     if (kr == FL(-1.0))
-      kr = DFLT_KR;
+      kr = csound->oparms->kr_default;//  DFLT_KR;
     ksmps = (MYFLT)((int)(sr / kr + FL(0.5)));
     kr = sr / ksmps; /* VL - avoid inconsistency */
   } else if (kr == FL(-1.0)) {
     if (sr == FL(-1.0))
-      sr = DFLT_SR;
+      sr = csound->oparms->sr_default;//  DFLT_SR;
     kr = sr / ksmps;
   } else if (sr == FL(-1.0)) {
     sr = kr * ksmps;
@@ -614,15 +615,17 @@ INSTRTXT *create_instrument0(CSOUND *csound, TREE *root,
                  "sr = %.7g, kr = %.7g, ksmps = %.7g", sr, kr,
                  ksmps);
     }
-    else if (UNLIKELY(FLOAT_COMPARE(sr, (double)kr * ksmps)))
+    else if (UNLIKELY(FLOAT_COMPARE(sr, (double)kr * ksmps)
+                      && !(O->ksmps_override || O->sr_override ||
+                            O->kr_override)))
       synterr(p, Str("%s inconsistent sr, kr, ksmps\n"), err_msg);
-    else if (UNLIKELY(ksmps > sr))
+    else if (UNLIKELY((ksmps > sr) && !(O->ksmps_override || O->sr_override)))
       synterr(p, Str("%s inconsistent sr, kr, ksmps\n"), err_msg);
   }
 
   csound->ksmps = ksmps;
   csound->nchnls = nchnls;
-  if (inchnls == 0)
+  if (inchnls < 0)
     csound->inchnls = nchnls;
   else
     csound->inchnls = inchnls;
@@ -639,13 +642,12 @@ INSTRTXT *create_instrument0(CSOUND *csound, TREE *root,
     csound->A4 = A4;
     csound_aops_init_tables(csound);
   }
-  if (UNLIKELY(csound->e0dbfs <= FL(0.0))) {
+  /* if (UNLIKELY(csound->e0dbfs <= FL(0.0))) {
     csound->Warning(csound, Str("bad value for 0dbfs: must be positive. "
                                 "Setting default value."));
     csound->e0dbfs = DFLT_DBFS;
-  }
+    }*/
 
-  OPARMS *O = csound->oparms;
   if (O->nchnls_override > 0)
     csound->nchnls = csound->inchnls = O->nchnls_override;
   if (O->nchnls_i_override > 0)
@@ -663,10 +665,15 @@ INSTRTXT *create_instrument0(CSOUND *csound, TREE *root,
     MYFLT tmp_sr = csound->esr;
     csound->esr = -1.0;
     O->sr_override = csoundInitialiseIO(csound);
-    if(O->sr_override > 0)
-     csound->Message(csound, "Using system sampling rate %.1f\n", O->sr_override);
+
+    if(O->sr_override > 0) {
+      if(O->msglevel || O->odebug)
+        csound->Message(csound,
+                        Str("Using system sampling rate %.1f\n"), O->sr_override);
+    }
     else {
-      csound->Message(csound, "System sr not available\n");
+      if(O->msglevel || O->odebug)
+        csound->Message(csound, Str("System sr not available\n"));
       O->sr_override = FL(0.0);
     }
     csound->esr = tmp_sr;
@@ -697,7 +704,7 @@ INSTRTXT *create_instrument0(CSOUND *csound, TREE *root,
       csound->ksmps = (ensmps = O->ksmps_override);
       if (O->sr_override) {
         csound->ekr = O->sr_override / csound->ksmps;
-        csound->esr = O->sr_override;
+        csound->esr = csound->ekr * ensmps; // force SR
       } else if (O->kr_override) {
         csound->esr = O->kr_override * csound->ksmps;
         csound->ekr = O->kr_override;
@@ -728,6 +735,7 @@ INSTRTXT *create_instrument0(CSOUND *csound, TREE *root,
       if (UNLIKELY(FLOAT_COMPARE(csound->esr, (double)csound->ekr * ensmps)))
         csoundDie(csound, Str("%s inconsistent sr, kr, ksmps"), s);
     }
+    if(O->odebug)
     csound->Message(csound, Str("sample rate overrides: "
                                 "esr = %7.4f, ekr = %7.4f, ksmps = %d\n"),
                     csound->esr, csound->ekr, csound->ksmps);
@@ -750,6 +758,7 @@ INSTRTXT *create_instrument0(CSOUND *csound, TREE *root,
   }
   close_instrument(csound, engineState, ip);
 
+  csound->inZero = 0;
   return ip;
 }
 
@@ -766,7 +775,7 @@ INSTRTXT *create_global_instrument(CSOUND *csound, TREE *root,
   OPTXT *op;
   TREE *current;
 
-  // csound->inZero = 1;
+  //csound->inZero = 1;
   find_or_add_constant(csound, engineState->constantsPool, "0", 0);
 
   ip = (INSTRTXT *)csound->Calloc(csound, sizeof(INSTRTXT));
@@ -813,7 +822,7 @@ INSTRTXT *create_global_instrument(CSOUND *csound, TREE *root,
   }
 
   close_instrument(csound, engineState, ip);
-  // csound->inZero = 0;
+  //csound->inZero = 0;
   return ip;
 }
 
@@ -1056,8 +1065,8 @@ void add_to_deadpool(CSOUND *csound, INSTRTXT *instrtxt) {
                     csound->dead_instr_no - 1);
 }
 
-int32 named_instr_find(CSOUND *csound, char *s);
-int32  named_instr_find_in_engine(CSOUND *csound, char *s,
+MYFLT named_instr_find(CSOUND *csound, char *s);
+MYFLT named_instr_find_in_engine(CSOUND *csound, char *s,
                                   ENGINE_STATE *engineState);
 /**
    allocate entry for named instrument ip with name s
@@ -1180,18 +1189,26 @@ void named_instr_assign_numbers(CSOUND *csound, ENGINE_STATE *engineState) {
 
   while (--insno_priority > -3) {
     if (insno_priority == -2) {
+      int found = 0;
       /* check both this state & current state */
-      num = engineState->maxinsno >
-        csound->engineState.maxinsno ?
-        engineState->maxinsno :
-        csound->engineState.maxinsno; /* find last used instr number */
-
-      /* check both this state & current state */
-      while ((!engineState->instrtxtp[num] ||
-              !csound->engineState.instrtxtp[num]) &&
-             --num)
-        ;
-
+      if(engineState->maxinsno > csound->engineState.maxinsno) {
+        for(num=engineState->maxinsno; num > csound->engineState.maxinsno; num--) {
+          if(engineState->instrtxtp[num]) {
+            found = 1;
+            break;
+          }
+        }
+      } else {
+        for(num=csound->engineState.maxinsno; num > engineState->maxinsno; num--) {
+          if(csound->engineState.instrtxtp[num]) {
+            found = 1;
+            break;
+          }
+        }
+      }
+      if(!found) {
+        while(!engineState->instrtxtp[num] && !csound->engineState.instrtxtp[num] && --num);
+      }
     }
     for (inm = inm_first; inm; inm = inm->next) {
       INSTRNAME *temp = (INSTRNAME *)inm->name;
@@ -1221,23 +1238,23 @@ void named_instr_assign_numbers(CSOUND *csound, ENGINE_STATE *engineState) {
             engineState->instrtxtp[m] = NULL;
         }
         inum = num;
-      } else
+
+      } else {
         inum = no; // else use existing number
+      }
       /* hack: "name" actually points to the corresponding INSTRNAME */
       inm2 = (INSTRNAME *)(inm->name); /* entry in the table */
 
       inm2->instno = (int32)inum;
       engineState->instrtxtp[inum] = inm2->ip;
 
-      //if(&csound->engineState == engineState) {
-        /* print message only after merge */
-       if (UNLIKELY((csound->oparms->odebug) || (csound->oparms->msglevel > 0)))
+      if (UNLIKELY((csound->oparms->odebug) || (csound->oparms->msglevel > 0)))
         csound->Message(csound, Str("instr %s uses instrument number %d\n"),
                         inm2->name, inum);
-       //}
     }
   }
   /* clear temporary chains */
+
   inm = inm_first;
   while (inm) {
     INSTRNAME *nxtinm = inm->next;
@@ -1626,8 +1643,10 @@ int csoundCompileTreeInternal(CSOUND *csound, TREE *root, int async) {
         csound, (1 + engineState->maxinsno) * sizeof(INSTRTXT *));
     /* VL: allowing global code to be evaluated in
        subsequent compilations */
+    //csound->inZero = 1;
     csound->instr0 = create_global_instrument(csound, current, engineState,
                                               typeTable->instr0LocalPool);
+    //csound->inZero = 0;
 
     insert_instrtxt(csound, csound->instr0, 0, engineState, 1);
 
@@ -1690,7 +1709,7 @@ int csoundCompileTreeInternal(CSOUND *csound, TREE *root, int async) {
 
         named_instr_alloc(csound, c, instrtxt, insno_priority, engineState, 0);
         if(engineState != &csound->engineState)
-        named_instr_assign_numbers(csound, engineState);
+          named_instr_assign_numbers(csound, engineState);
         /* VL 10.10.14: check for redefinition */
         // if (UNLIKELY(!named_instr_alloc(csound, c,
         //  instrtxt, insno_priority,
@@ -1729,9 +1748,9 @@ int csoundCompileTreeInternal(CSOUND *csound, TREE *root, int async) {
               // VL 25.05.2018
               // this should only be run here in the
               // first compilation
-              //if(engineState == &csound->engineState)
-              //named_instr_alloc(csound, c, instrtxt, insno_priority,
-              //                engineState, 0);
+              if(engineState == &csound->engineState)
+                named_instr_alloc(csound, c, instrtxt, insno_priority,
+                                  engineState, 0);
               /* if (UNLIKELY(!named_instr_alloc(csound, c, */
               /*                                 instrtxt, insno_priority, */
               /*                                 engineState,0))) { */
@@ -1828,14 +1847,15 @@ int csoundCompileTreeInternal(CSOUND *csound, TREE *root, int async) {
     return CSOUND_ERROR;
   }
 
-  /* now add the instruments with names, assigning them fake instr numbers */
-  named_instr_assign_numbers(csound, engineState);
+
   if (engineState != &csound->engineState) {
     OPDS *ids = csound->ids;
     /* any compilation other than the first one */
     /* merge ENGINE_STATE */
     /* lock to ensure thread-safety */
     if (!async) {
+      /* now add the instruments with names, assigning them fake instr numbers */
+      named_instr_assign_numbers(csound, engineState);
       if (!csound->oparms->realtime)
         csoundLockMutex(csound->API_lock);
       merge_state(csound, engineState, typeTable, ids);
@@ -1849,6 +1869,8 @@ int csoundCompileTreeInternal(CSOUND *csound, TREE *root, int async) {
         csoundSpinUnLock(&csound->alloc_spinlock);
     }
   } else {
+    /* now add the instruments with names, assigning them fake instr numbers */
+    named_instr_assign_numbers(csound, engineState);
     /* first compilation */
     insert_opcodes(csound, csound->opcodeInfo, engineState);
     ip = engineState->instxtanchor.nxtinstxt;
@@ -1932,8 +1954,10 @@ int csoundCompileOrcInternal(CSOUND *csound, const char *str, int async) {
   root = csoundParseOrc(csound, str);
   if (LIKELY(root != NULL)) {
     retVal = csoundCompileTreeInternal(csound, root, async);
+#ifdef PARCS
     // Sanitise semantic sets here
     sanitize(csound);
+#endif
     csoundDeleteTree(csound, root);
   } else {
     // csoundDeleteTree(csound, root);
@@ -1949,7 +1973,8 @@ int csoundCompileOrcInternal(CSOUND *csound, const char *str, int async) {
 
 /* prep an instr template for efficient allocs  */
 /* repl arg refs by offset ndx to lcl/gbl space */
-static void insprep(CSOUND *csound, INSTRTXT *tp, ENGINE_STATE *engineState) {
+static void insprep(CSOUND *csound, INSTRTXT *tp, ENGINE_STATE *engineState)
+{
   OPARMS *O = csound->oparms;
   OPTXT *optxt;
   OENTRY *ep;
@@ -2036,11 +2061,14 @@ static void insprep(CSOUND *csound, INSTRTXT *tp, ENGINE_STATE *engineState) {
         ARG *inArgs = ttp->inArgs;
         // CS_VARIABLE* var;
 
-        if (tp->insname)
-          csound->Message(csound, "PSET: isname=\"%s\", pmax=%d\n", tp->insname,
-                          tp->pmax);
-        else
-          csound->Message(csound, "PSET: isno=??, pmax=%d\n", tp->pmax);
+        if (tp->insname) {
+          if(csound->oparms_.msglevel || csound->oparms_.odebug)
+              csound->Message(csound, "PSET: isname=\"%s\", pmax=%d\n", tp->insname,
+                              tp->pmax);
+        } else {
+          if(csound->oparms_.msglevel || csound->oparms_.odebug)
+            csound->Message(csound, "PSET: isno=??, pmax=%d\n", tp->pmax);
+        }
         if (UNLIKELY((n = ttp->inArgCount) != tp->pmax)) {
           // csound->Warning(csound, Str("i%d pset args != pmax"), (int) insno);
           csound->Warning(csound, Str("i[fixme] pset args != pmax"));
@@ -2324,7 +2352,8 @@ void debugPrintCsound(CSOUND *csound) {
 void query_deprecated_opcode(CSOUND *csound, ORCTOKEN *o) {
     char *name = o->lexeme;
     OENTRY *ep = find_opcode(csound, name);
-    if (UNLIKELY((ep->flags &_QQ) && !(csound->oparms_.msglevel&NOQQ)))
+    if (UNLIKELY((ep->flags &_QQ) &&
+                 (csound->oparms_.msglevel&CS_NOQQ)==CS_NOQQ))
       csound->Warning(csound, Str("Opcode \"%s\" is deprecated\n"), name);
 }
 
