@@ -29,10 +29,9 @@
    1 =     1
    2 =             2
    3 =     1  AND  2
- */
+*/
 
 /* inarg types include the following:
-
    i       irate scalar
    k       krate scalar
    a       arate vector
@@ -64,7 +63,8 @@
    W       begins indef list of Strings (any count)
    y       begins indef list of aargs (any count)
    z       begins indef list of kargs (any count)
-   Z       begins alternating kakaka...list (any count)    */
+   Z       begins alternating kakaka...list (any count)
+*/
 
 /* outarg types include:
  i, k, a, S as  above
@@ -76,8 +76,12 @@
  X       multiple args (a, k, or i-rate)     IV - Sep 1 2002
  N       multiple args (a, k, i, or S-rate)
  F       multiple args (f-rate)#
- */
+*/
 
+/* inargs and outargs may also be arrays, e.g. "a[]" is an array of
+   arate vectors. Then for polymorphic opcode entries, "opcode.a" is
+   for arate vectors, and "opcode.A" is for arrays of arate vectors.
+*/
 
 OENTRY opcodlst_1[] = {
   /* opcode   dspace  flags  thread  outarg  inargs  isub    ksub    asub    */
@@ -107,6 +111,15 @@ OENTRY opcodlst_1[] = {
   { "xout", S(XOUT_MAX),0,  1,  "",         "*", xoutset, NULL, NULL, NULL },
   { "setksmps", S(SETKSMPS),0,  1,  "",   "i", setksmpsset, NULL, NULL },
   { "ctrlinit",S(CTLINIT),0,1,      "",  "im", ctrlinit, NULL, NULL, NULL},
+  { "ctrlinit.S",S(CTLINITS),0,1,      "",  "Sm", ctrlnameinit, NULL, NULL, NULL},
+  { "ctrlsave",S(SAVECTRL),0,3,       "k[]","im", savectrl_init, savectrl_perf, NULL, NULL},
+  { "ctrlprint.S",S(PRINTCTRL),0,3, "", "k[]S", printctrl_init1, printctrl, NULL},
+  { "ctrlprint",S(PRINTCTRL),0,3,       "", "k[]", printctrl_init, printctrl, NULL},
+  { "ctrlpreset", S(PRESETCTRL1), 0,3, "k", "kk[]", presetctrl1_init, presetctrl1_perf, NULL},
+  { "ctrlpreset", S(PRESETCTRL), 0,3, "k", "kim", presetctrl_init, presetctrl_perf, NULL},
+  { "ctrlselect", S(SELECTCTRL), 0,3,"",   "k", selectctrl_init, selectctrl_perf, NULL },
+  { "ctrlprintpresets", S(PRINTPRESETS), 0,3, "", "", printpresets_init, printpresets_perf, NULL},
+  { "ctrlprintpresets.S", S(PRINTPRESETS), 0,3, "", "S", printpresets_init1, printpresets_perf, NULL},
   { "massign",S(MASSIGN), 0,1,      "",  "iip",massign_p, NULL, NULL, NULL},
   { "massign.iS",S(MASSIGNS), 0,1,  "",  "iSp",massign_S, NULL, NULL, NULL},
   { "turnon", S(TURNON),  0,1,      "",     "io", turnon, NULL, NULL, NULL},
@@ -120,6 +133,8 @@ OENTRY opcodlst_1[] = {
   { "turnoff",S(LINK),0,    2,      "",     "",     NULL,   turnoff, NULL, NULL },
   {  "=.S",   S(STRCPY_OP),0,   1,  "S",    "S",
      (SUBR) strcpy_opcode_S, NULL, (SUBR) NULL, NULL    },
+  {  "#=.S",   S(STRCPY_OP),0,   2,  "S",    "S",
+     NULL, (SUBR) strcpy_opcode_S, (SUBR) NULL, NULL    },
   {  "=.T",   S(STRGET_OP),0,   1,  "S",    "i",
      (SUBR) strcpy_opcode_p, (SUBR) NULL, (SUBR) NULL, NULL                 },
   { "=.r",    S(ASSIGN),0,  1,      "r",    "i",    rassign, NULL, NULL, NULL },
@@ -204,6 +219,7 @@ OENTRY opcodlst_1[] = {
   { "round.i",S(EVAL),0,    1,      "i",    "i",    int1_round              },
   { "floor.i",S(EVAL),0,    1,      "i",    "i",    int1_floor              },
   { "ceil.i", S(EVAL),0,    1,      "i",    "i",    int1_ceil               },
+  { "rndseed", S(EVAL),0,    1,      "",    "i",    rnd1seed                },
   { "rnd.i",  S(EVAL),0,    1,      "i",    "i",    rnd1                    },
   { "birnd.i",S(EVAL),0,    1,      "i",    "i",    birnd1                  },
   { "abs.i",  S(EVAL),0,    1,      "i",    "i",    abs1                    },
@@ -373,41 +389,35 @@ OENTRY opcodlst_1[] = {
   { "signum.i", S(ASSIGN), 0, 1,     "i",   "i", signum, NULL, NULL       },
   { "signum.k", S(ASSIGN), 0, 3,     "k",   "k", signum, signum, NULL     },
   { "signum.a", S(ASSIGN), 0, 2,     "a",   "a", NULL, asignum      },
-  /* { "table",  0xffff, TR                                                   }, */
-  /* { "tablei", 0xffff, TR                                                   }, */
-  /* { "table3", 0xffff, TR                                                   }, */
   { "table.i",  S(TABL),TR, 1,      "i",    "iiooo",(SUBR)tabler_init       },
   { "table.k",  S(TABL),TR, 3,      "k",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tabler_kontrol        },
   { "table.a",  S(TABL),TR, 3,      "a",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tabler_audio                                                      },
-  { "tablei.i", S(TABL),TR, 1,      "i",    "iiooo",(SUBR)tableir_init       },
+  { "tablei.i", S(TABL),TR, 1,      "i",    "iiooo",(SUBR)tableir_init      },
   { "tablei.k", S(TABL),TR, 3,      "k",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tableir_kontrol                                                   },
   { "tablei.a", S(TABL),TR, 3,      "a",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tableir_audio                                                     },
-  { "table3.i", S(TABL),TR, 1,      "i",    "iiooo",(SUBR)table3r_init       },
+  { "table3.i", S(TABL),TR, 1,      "i",    "iiooo",(SUBR)table3r_init      },
   { "table3.k", S(TABL),TR, 3,      "k",    "xiooo",(SUBR)tabl_setup,
     (SUBR)table3r_kontrol                                                   },
   { "table3.a", S(TABL),TR, 3,      "a",    "xiooo",(SUBR)tabl_setup,
     (SUBR)table3r_audio                                                     },
-  /* { "ptable",  0xffff, TR                                                  }, */
-  /* { "ptablei", 0xffff, TR                                                  }, */
-  /* { "ptable3", 0xffff, TR                                                  }, */
-  { "ptable.i",  S(TABLE),TR, 1,     "i",    "iiooo",(SUBR)tabler_init       },
-  { "ptable.k",  S(TABLE),TR, 3,     "k",    "xiooo",(SUBR)tabl_setup,
+  { "ptable.i",  S(TABLE),TR|_QQ, 1,"i",    "iiooo",(SUBR)tabler_init       },
+  { "ptable.k",  S(TABLE),TR|_QQ, 3,     "k",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tabler_kontrol                                                    },
-  { "ptable.a",  S(TABLE),TR, 3,     "a",    "xiooo",(SUBR)tabl_setup,
+  { "ptable.a",  S(TABLE),TR|_QQ, 3,     "a",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tabler_audio                                                      },
-  { "ptablei.i", S(TABLE),TR, 1,     "i",    "iiooo",(SUBR)tableir_init      },
-  { "ptablei.k", S(TABLE),TR, 3,     "k",    "xiooo",(SUBR)tabl_setup,
+  { "ptablei.i", S(TABLE),TR|_QQ, 1,"i",    "iiooo",(SUBR)tableir_init      },
+  { "ptablei.k", S(TABLE),TR|_QQ, 3,     "k",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tableir_kontrol                                                   },
-  { "ptablei.a", S(TABLE),TR, 3,     "a",    "xiooo",(SUBR)tabl_setup,
+  { "ptablei.a", S(TABLE),TR|_QQ, 3,     "a",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tableir_audio                                                     },
-  { "ptable3.i", S(TABLE),TR, 1,     "i",    "iiooo",(SUBR)table3r_init      },
-  { "ptable3.k", S(TABLE),TR, 3,     "k",    "xiooo",(SUBR)tabl_setup,
+  { "ptable3.i", S(TABLE),TR|_QQ, 1,"i",    "iiooo",(SUBR)table3r_init      },
+  { "ptable3.k", S(TABLE),TR|_QQ, 3,     "k",    "xiooo",(SUBR)tabl_setup,
     (SUBR)table3r_kontrol                                                   },
-  { "ptable3.a", S(TABLE),TR, 3,     "a",    "xiooo",(SUBR)tabl_setup,
+  { "ptable3.a", S(TABLE),TR|_QQ, 3,     "a",    "xiooo",(SUBR)tabl_setup,
     (SUBR)table3r_audio         },
   { "oscil1", S(OSCIL1), TR, 3,     "k",    "ikij", ko1set, kosc1          },
   { "oscil1i",S(OSCIL1), TR, 3,     "k",    "ikij", ko1set, kosc1i         },
@@ -518,6 +528,7 @@ OENTRY opcodlst_1[] = {
   { "outq2",  S(OUTM),IR,    3,      "",     "a",    och2,   outs2   },
   { "outq3",  S(OUTM),IR,    3,      "",     "a",    och3,   outq3   },
   { "outq4",  S(OUTM),IR,    3,      "",     "a",    och2,   outq4   },
+  { "outall", S(OUTM),IR,    2,      "",     "a",    NULL,   outrep  },
   { "igoto",  S(GOTO),0,    1,      "",     "l",    igoto                   },
   { "kgoto",  S(GOTO),0,    2,      "",     "l",    NULL,   kgoto           },
   { "goto",   S(GOTO),0,    3,      "",     "l",    igoto,  kgoto           },
@@ -588,6 +599,9 @@ OENTRY opcodlst_1[] = {
   { "gauss.i" , S(PRAND),0, 1,      "i",    "k",    ikgaus,  NULL,  NULL    },
   { "gauss.k" , S(PRAND),0, 2,      "k",    "k",    NULL, ikgaus,   NULL    },
   { "gauss.a" , S(PRAND),0, 2,      "a",    "k",    NULL,  agaus   },
+  { "gauss.iii" , S(GAUSS),0, 1,      "i",    "ii",    gauss_scalar,  NULL,  NULL    },
+  { "gauss.kkk" , S(GAUSS),0, 2,      "k",    "kk",    NULL, gauss_scalar,   NULL    },
+  { "gauss.akk" , S(GAUSS),0, 2,      "a",    "kk",    NULL, gauss_vector   },
   { "weibull.i",S(PRAND),0, 1,      "i",    "kk",   ikweib,  NULL,  NULL    },
   { "weibull.k",S(PRAND),0, 2,      "k",    "kk",   NULL, ikweib,   NULL    },
   { "weibull.a",S(PRAND),0, 2,      "a",    "kk",   NULL,  aweib   },
@@ -679,6 +693,8 @@ OENTRY opcodlst_1[] = {
   { "filebit.i", S(SNDINFO),0,  1,     "i",   "ip",   filebit, NULL, NULL        },
   { "filepeak.i", S(SNDINFOPEAK),0, 1, "i",   "io",   filepeak, NULL, NULL       },
   { "filevalid.i", S(FILEVALID),0, 1,  "i",   "i",    filevalid, NULL, NULL      },
+  { "filevalid.k", S(FILEVALID),0, 2,  "k",   "S",    NULL, filevalid_S, NULL    },
+  { "filevalid.k", S(FILEVALID),0, 2,  "k",   "i",    NULL, filevalid, NULL    },
   /*  { "nlalp", S(NLALP),0,     3,     "a",  "akkoo", nlalp_set, nlalp }, */
   { "ptableiw",  S(TABLEW),TW|_QQ, 1, "", "iiiooo", (SUBR)tablew_init, NULL, NULL},
   { "ptablew.kk", S(TABLEW),TB,  3,  "", "kkiooo",(SUBR)tabl_setup,
@@ -795,6 +811,8 @@ OENTRY opcodlst_1[] = {
     schedule_S, NULL, NULL },
   { "schedule.SN", S(SCHED),0,  1,     "",     "SiiN",
     schedule_SN, NULL, NULL },
+  { "schedule.array", S(SCHED),0,  1,     "",     "i[]",
+    schedule_array, NULL, NULL },
   /* **** Experimental schedulek opcodes **** */
   { "schedulek",   S(SCHED),0,  2,     "",     "kkkM",
     NULL, schedule, NULL },
@@ -804,6 +822,9 @@ OENTRY opcodlst_1[] = {
     NULL, schedule_S, NULL },
   { "schedulek.SN", S(SCHED),0, 2,     "",     "SkkN",
     NULL, schedule_SN, NULL },
+  { "schedulek.array",   S(SCHED),0,  2,     "",     "k[]",
+    NULL, schedule_array, NULL },
+
   /* **** End of schedulek **** */
   { "schedwhen", S(WSCHED),0,3,     "",     "kkkkm",ifschedule, kschedule, NULL },
   { "schedwhen", S(WSCHED),0,3,     "",     "kSkkm",ifschedule, kschedule, NULL },
@@ -836,24 +857,18 @@ OENTRY opcodlst_1[] = {
     (SUBR)reverbx_set,(SUBR)reverbx  },
   { "nreverb",  S(NREV2),0,  3,     "a",    "akkoojoj",
     (SUBR)reverbx_set,(SUBR) reverbx },
-  { "=.f",      S(FASSIGN),0, 3,    "f",   "f",
-    (SUBR)fassign_set, (SUBR)fassign },
-  { "init.f",   S(FASSIGN),0, 1,    "f",   "f",
-    (SUBR)fassign_set, NULL, NULL    },
-  { "pvsanal",  S(PVSANAL), 0, 3,   "f",   "aiiiioo",
-    pvsanalset, pvsanal   },
+  { "=.f",      S(FASSIGN),0, 3,    "f",   "f", (SUBR)fassign_set, (SUBR)fassign },
+  { "init.f",   S(FASSIGN),0, 1,    "f",   "f", (SUBR)fassign_set, NULL, NULL    },
+  { "pvsanal",  S(PVSANAL), 0, 3,   "f",   "aiiiioo", pvsanalset, pvsanal   },
   { "pvsynth",  S(PVSYNTH),0, 3,    "a",   "fo",     pvsynthset, pvsynth },
   { "pvsadsyn", S(PVADS),0,   3,    "a",   "fikopo", pvadsynset, pvadsyn, NULL },
   { "pvscross", S(PVSCROSS),0,3,    "f",   "ffkk",   pvscrosset, pvscross, NULL },
   { "pvsfread", S(PVSFREAD),0,3,    "f",   "kSo",    pvsfreadset_S, pvsfread, NULL},
-  { "pvsfread.i", S(PVSFREAD),0,3,    "f",   "kio",    pvsfreadset, pvsfread, NULL},
+  { "pvsfread.i", S(PVSFREAD),0,3,  "f",   "kio",    pvsfreadset, pvsfread, NULL},
   { "pvsmaska", S(PVSMASKA),0,3,    "f",   "fik",    pvsmaskaset, pvsmaska, NULL  },
-  { "pvsftw",   S(PVSFTW),  TW, 3,    "k",   "fio",    pvsftwset, pvsftw, NULL  },
+  { "pvsftw",   S(PVSFTW),  TW, 3,  "k",   "fio",    pvsftwset, pvsftw, NULL  },
   { "pvsftr",   S(PVSFTR),TR, 3,    "",    "fio",    pvsftrset, pvsftr, NULL  },
   { "pvsinfo",  S(PVSINFO),0, 1,    "iiii","f",      pvsinfo, NULL, NULL    },
-  /* { "octave",   0xffff                                                      }, */
-  /* { "semitone", 0xffff                                                      }, */
-  /* { "cent",     0xffff                                                      }, */
   { "octave.i", S(EVAL),0,    1,    "i",    "i",     powoftwo               },
   { "octave.k", S(EVAL),0,    2,    "k",    "k",     NULL,  powoftwo        },
   { "octave.a", S(EVAL),0,    2,    "a",    "a",     NULL, powoftwoa  },
@@ -863,13 +878,12 @@ OENTRY opcodlst_1[] = {
   { "cent.i",   S(EVAL),0,    1,    "i",    "i",     cent                   },
   { "cent.k",   S(EVAL),0,    2,    "k",    "k",     NULL,  cent            },
   { "cent.a",   S(EVAL),0,    2,    "a",    "a",     NULL, acent      },
-  //{ "db",       0xffff                                                      },
   { "db.i",     S(EVAL),0,    1,    "i",    "i",     db                     },
   { "db.k",     S(EVAL),0,    2,    "k",    "k",     NULL, db               },
   { "db.a",     S(EVAL),0,    2,    "a",    "a",     NULL, dba        },
   { "midichn",  S(MIDICHN),0, 1,    "i",    "",      midichn, NULL, NULL    },
   { "pgmassign",S(PGMASSIGN),0, 1,  "",     "iio",   pgmassign, NULL, NULL  },
-    { "pgmassign.S",S(PGMASSIGN),0, 1,  "",     "iSo",   pgmassign_S, NULL, NULL  },
+  { "pgmassign.S",S(PGMASSIGN),0, 1, "",    "iSo",   pgmassign_S, NULL, NULL  },
   { "midiin",   S(MIDIIN),0,  3,    "kkkk", "",      midiin_set, midiin, NULL },
   { "pgmchn",   S(PGMIN),0,  3,     "kk",   "o",     pgmin_set, pgmin, NULL },
   { "ctlchn",   S(CTLIN),0,  3,     "kkk",  "oo",    ctlin_set, ctlin, NULL },
@@ -877,7 +891,7 @@ OENTRY opcodlst_1[] = {
     (SUBR) midiTempoOpcode, (SUBR) midiTempoOpcode, NULL    },
   { "midifilestatus", S(MIDITEMPO),0, 2, "k",    "",
    NULL, (SUBR) midiFileStatus, NULL },
-  { "midinoteoff", S(MIDINOTEON),0,3,"",    "xx",   midinoteoff, midinoteoff, },
+  { "midinoteoff", S(MIDINOTEON),0,3   ,"", "xx",   midinoteoff, midinoteoff, },
   { "midinoteonkey", S(MIDINOTEON),0,3, "", "xx",   midinoteonkey, midinoteonkey },
   { "midinoteoncps", S(MIDINOTEON),0, 3, "", "xx",  midinoteoncps,midinoteoncps },
   { "midinoteonoct", S(MIDINOTEON),0, 3, "", "xx",  midinoteonoct,midinoteonoct },
@@ -896,18 +910,18 @@ OENTRY opcodlst_1[] = {
   //{ "invalue",   0xFFFF,   _CR,    0,   NULL,   NULL, NULL, NULL },
   { "invalue.k", S(INVAL),_CR, 3, "k", "i", (SUBR) invalset,(SUBR)kinval, NULL },
   { "invalue.i", S(INVAL),_CR, 1, "i", "i", (SUBR) invalsetgo, NULL, NULL },
-  { "invalue.kS", S(INVAL),_CR, 3, "k", "S",(SUBR) invalset_S,(SUBR)kinval, NULL },
   { "invalue.iS", S(INVAL),_CR, 1, "i", "S", (SUBR) invalsetSgo, NULL, NULL },
+  { "invalue.kS", S(INVAL),_CR, 3, "k", "S",(SUBR) invalset_S,(SUBR)kinval, NULL },
   { "invalue.S", S(INVAL),_CR, 3, "S", "i",
                                       (SUBR) invalset_string, (SUBR)kinvalS, NULL },
   { "invalue.SS", S(INVAL),_CR, 3, "S", "S",
                                     (SUBR) invalset_string_S, (SUBR)kinvalS, NULL },
   { "outvalue", S(OUTVAL), _CW, 3, "", "ik", (SUBR) outvalset, (SUBR)koutval, NULL},
   { "outvalue.i", S(OUTVAL), _CW, 1, "", "ii", (SUBR) outvalsetgo, NULL, NULL},
-  { "outvalue.k", S(OUTVAL), _CW, 3, "", "Sk",
-                                          (SUBR) outvalset_S, (SUBR)koutval, NULL},
   { "outvalue.Si", S(OUTVAL), _CW, 1, "", "Si",
                                           (SUBR) outvalsetSgo, NULL, NULL},
+  { "outvalue.k", S(OUTVAL), _CW, 3, "", "Sk",
+                                          (SUBR) outvalset_S, (SUBR)koutval, NULL},
   { "outvalue.S", S(OUTVAL), _CW, 3, "", "iS",
                                      (SUBR) outvalset_string, (SUBR)koutvalS, NULL},
   { "outvalue.SS", S(OUTVAL), _CW, 3, "", "SS",
@@ -922,11 +936,18 @@ OENTRY opcodlst_1[] = {
   { "nstrstr", S(NSTRSTR),0, 1,       "S",    "i",    nstrstr, NULL, NULL      },
   { "nstrstr.k", S(NSTRSTR),0, 2,     "S",    "k",    NULL, nstrstr, NULL      },
   //{ "turnoff2",   0xFFFB,   _CW,    0, NULL,   NULL,   NULL, NULL, NULL          },
+  { "turnoff2_i.S",S(TURNOFF2),_CW,1,     "",     "Soo",  turnoff2S, NULL     },
+  { "turnoff2_i.i",S(TURNOFF2),_CW,1,     "",     "ioo",  turnoff2k, NULL     },
   { "turnoff2.S",S(TURNOFF2),_CW,2,     "",     "Skk",  NULL, turnoff2S, NULL     },
   { "turnoff2.c",S(TURNOFF2),_CW,2,     "",     "ikk",  NULL, turnoff2k, NULL     },
   { "turnoff2.k",S(TURNOFF2),_CW,2,     "",     "kkk",  NULL, turnoff2k, NULL     },
   { "turnoff2.i",S(TURNOFF2),_CW,2,     "",     "ikk",  NULL, turnoff2k, NULL     },
   { "turnoff2.r",S(TURNOFF2),_CW,2,     "",     "ikk",  NULL, turnoff2k, NULL     },
+  { "turnoff3.S",S(TURNOFF2),_CW,2,     "",     "S",  NULL, turnoff3S, NULL     },
+  { "turnoff3.c",S(TURNOFF2),_CW,2,     "",     "i",  NULL, turnoff3k, NULL     },
+  { "turnoff3.k",S(TURNOFF2),_CW,2,     "",     "k",  NULL, turnoff3k, NULL     },
+  { "turnoff3.i",S(TURNOFF2),_CW,2,     "",     "i",  NULL, turnoff3k, NULL     },
+  { "turnoff3.r",S(TURNOFF2),_CW,2,     "",     "i",  NULL, turnoff3k, NULL     },
   { "cngoto", S(CGOTO),0,   3,      "",     "Bl",   ingoto, kngoto, NULL     },
   { "cnkgoto", S(CGOTO),0,   2,      "",     "Bl",   NULL,  kngoto, NULL     },
   { "cingoto", S(CGOTO),0,   1,      "",     "Bl",   ingoto, NULL, NULL     },
@@ -1056,14 +1077,10 @@ OENTRY opcodlst_1[] = {
 #ifdef HAVE_CURL
   {  "strfromurl", S(STRCPY_OP), 0, 1, "S", "S", (SUBR) str_from_url     },
 #endif
-  {  "changed.S", S(STRCHGD),_QQ, 3, "k",   "S",
+  {  "changed.S", S(STRCHGD),0, 3, "k",   "S",
      (SUBR) str_changed, (SUBR) str_changed_k, NULL       },
   {  "changed2.S", S(STRCHGD),0, 3, "k",   "S",
      (SUBR) str_changed, (SUBR) str_changed_k, NULL       },
-  /* { "loop_lt",   0xfffb                                                  }, */
-  /* { "loop_le",   0xfffb                                                  }, */
-  /* { "loop_gt",   0xfffb                                                  }, */
-  /* { "loop_ge",   0xfffb                                                  }, */
   { "loop_lt.i", S(LOOP_OPS),0,  1,  "", "iiil", (SUBR) loop_l_i, NULL, NULL   },
   { "loop_le.i", S(LOOP_OPS),0,  1,  "", "iiil", (SUBR) loop_le_i, NULL, NULL  },
   { "loop_gt.i", S(LOOP_OPS),0,  1,  "", "iiil", (SUBR) loop_g_i, NULL, NULL   },
@@ -1122,6 +1139,8 @@ OENTRY opcodlst_1[] = {
     (SUBR) chnclear_opcode_init, (SUBR) notinit_opcode_stub },
   { "chn_k",       S(CHN_OPCODE_K),    _CW, 1,      "",             "SiooooooooN",
     (SUBR) chn_k_opcode_init, NULL, NULL                  },
+  { "chn_k",       S(CHN_OPCODE_K),    _CW, 1,      "",             "SSooooooooN",
+    (SUBR) chn_k_opcode_init_S, NULL, NULL},
   { "chn_a",       S(CHN_OPCODE),      _CW, 1,      "",             "Si",
     (SUBR) chn_a_opcode_init, NULL, NULL                  },
   { "chn_S",       S(CHN_OPCODE),      _CW, 1,      "",             "Si",
@@ -1195,6 +1214,26 @@ OENTRY opcodlst_1[] = {
     (SUBR)hw_channels},
    { "midiarp",   S(MIDIARP),0,  3,    "kk", "kO",
      midiarp_set, midiarp, NULL },
+   {"lpcfilter", S(LPCFIL), 0, 3, "a", "akkiiio",
+   (SUBR) lpfil_init, (SUBR) lpfil_perf},
+   {"lpcfilter", S(LPCFIL2), 0, 3, "a", "aakkiio",
+   (SUBR) lpfil2_init, (SUBR) lpfil2_perf},
+   {"allpole", S(LPCFIL3), 0, 3, "a", "ak[]",
+   (SUBR) lpfil3_init, (SUBR) lpfil3_perf},
+   {"lpcanal", S(LPREDA), 0, 3, "k[]kkk", "kkiiio",
+   (SUBR) lpred_alloc, (SUBR) lpred_run},
+   {"lpcanal", S(LPREDA2), 0, 3, "k[]kkk", "akkiio",
+   (SUBR) lpred_alloc2, (SUBR) lpred_run2},
+   {"lpcanal", S(LPREDA), 0, 1, "i[]iii", "iiii",
+   (SUBR) lpred_i, NULL},
+   {"pvslpc", S(LPCPVS), 0, 3, "f", "aiiio",
+    (SUBR) lpcpvs_init, (SUBR) lpcpvs},
+   {"pvscfs", S(PVSCFS), 0, 3, "k[]kk", "fip",
+    (SUBR) pvscoefs_init, (SUBR) pvscoefs},
+   {"apoleparams", S(CF2P), 0, 3, "k[]", "k[]",
+    (SUBR) coef2parm_init, (SUBR) coef2parm},
+   {"resonbnk", S(RESONB), 0, 3, "a", "ak[]kkipoo",
+   (SUBR) resonbnk_init, (SUBR) resonbnk},
   /* terminate list */
   {  NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL       }
 };
