@@ -62,12 +62,16 @@ static inline void alloc_globals(CSOUND *csound)
 
 static void spoutsf(CSOUND *csound)
 {
+    OPARMS  *O = csound->oparms;
     uint32_t   chn = 0;
     int     n;
     int spoutrem = csound->nspout;
     MYFLT   *sp = csound->spout;
     MYFLT   absamp = FL(0.0);
     uint32  nframes = csound->libsndStatics.nframes;
+    MYFLT lim = O->limiter*csound->e0dbfs;
+    MYFLT rlim = lim==0 ? 0 : FL(1.0)/lim;
+    MYFLT k1 = FL(1.0)/TANH(FL(1.0)); /*  1.31304 */
  nchk:
     /* if nspout remaining > buf rem, prepare to send in parts */
     if ((n = spoutrem) > (int) csound->libsndStatics.outbufrem) {
@@ -76,16 +80,38 @@ static void spoutsf(CSOUND *csound)
     spoutrem -= n;
     csound->libsndStatics.outbufrem -= n;
     do {
-      absamp = *sp++;
-      if (csound->libsndStatics.osfopen) {
-        *csound->libsndStatics.outbufp++ = (absamp * csound->dbfs_to_float);
+      // built inlimiter start ****
+      // There is a rather awkward problem in reporting out of range not being
+      // confused by the limited value but passing the clipped values to the
+      // output.  Current solution is nasty and should be easier
+      if (O->limiter) {
+        MYFLT x = *sp;
+        absamp = x;
+        if (UNLIKELY(x>=lim))
+          x = lim;
+        else if (UNLIKELY(x<= -lim))
+          x = -lim;
+        else
+          x = lim*k1*TANH(x*rlim);
+        //printf("*** %g -> %g\n", *(sp-1), x);
+        *sp++ = x;
+        if (csound->libsndStatics.osfopen) {
+          *csound->libsndStatics.outbufp++ = (x * csound->dbfs_to_float);
+        }
+      }
+      // limiter end ****
+      else {
+        absamp = *sp++;
+        if (csound->libsndStatics.osfopen) {
+          *csound->libsndStatics.outbufp++ = (absamp * csound->dbfs_to_float);
+        }
       }
       if (absamp < FL(0.0)) {
         absamp = -absamp;
       }
       if (absamp > csound->maxamp[chn]) {   /*  maxamp this seg  */
         csound->maxamp[chn] = absamp;
-        csound->maxpos[chn] = nframes;
+         csound->maxpos[chn] = nframes;
       }
       if (absamp > csound->e0dbfs) {        /* out of range?     */
         csound->rngcnt[chn]++;              /*  report it        */
@@ -476,6 +502,11 @@ void sfopenin(CSOUND *csound)           /* init for continuous soundin */
     int     fileType = (int) TYP_RAW;
     int     isfd = 0;   /* stdin */
 
+    if(csound->inchnls < 1)
+       csound->Die(csound,
+                 Str("error: cannot run input audio with nchnls_i=0"));
+
+
     alloc_globals(csound);
     STA(inbufrem) = (uint32) 0;    /* start with empty buffer */
     sfname = O->infilename;
@@ -571,13 +602,14 @@ void sfopenin(CSOUND *csound)           /* init for continuous soundin */
     STA(inbufsiz) = (unsigned) (O->inbufsamps * sizeof(MYFLT));
     STA(inbuf) = (MYFLT*) csound->Calloc(csound,
                                          STA(inbufsiz)); /* alloc inbuf space */
-    if (STA(pipdevout) == 2)
-      csound->Message(csound,
+    if (STA(pipdevout) == 2) {
+       csound->Message(csound,
                       Str("reading %d sample blks of %lu-bit floats from %s\n"),
                       O->inbufsamps * O->sfsampsize,
                       (unsigned long) sizeof(MYFLT)*8, sfname);
+    }
     else {
-      csound->Message(csound,
+       csound->Message(csound,
                       Str("reading %d-byte blks of %s from %s (%s)\n"),
                       O->inbufsamps * (int) sfsampsize(FORMAT2SF(O->informat)),
                       getstrformat(O->informat), sfname, type2string(fileType));
@@ -866,13 +898,15 @@ void sfopenout(CSOUND *csound)                  /* init for sound out       */
     /* calc outbuf size & alloc bufspace */
     STA(outbufsiz) = O->outbufsamps * sizeof(MYFLT);
     STA(outbufp)   = STA(outbuf) = csound->Malloc(csound, STA(outbufsiz));
-    if (STA(pipdevout) == 2)
+    if (STA(pipdevout) == 2) {
       csound->Message(csound,
                       Str("writing %d sample blks of %lu-bit floats to %s\n"),
                       O->outbufsamps, (unsigned long) sizeof(MYFLT)*8,
                       STA(sfoutname));
+
+    }
     else {
-     csound->Message(csound, Str("writing %d-byte blks of %s to %s"),
+      csound->Message(csound, Str("writing %d-byte blks of %s to %s"),
                     O->outbufsamps * O->sfsampsize,
                     getstrformat(O->outformat), STA(sfoutname));
 
@@ -956,6 +990,7 @@ void sfcloseout(CSOUND *csound)
         csound->Message(csound, Str(" (raw)\n"));
       else
         csound->Message(csound, " (%s)\n", type2string(O->filetyp));
+      
     }
     STA(osfopen) = 0;
 }
@@ -979,6 +1014,7 @@ void sfnopenout(CSOUND *csound)
 {
     alloc_globals(csound);
     csound->Message(csound, Str("not writing to sound disk\n"));
+    
     /* init counter, though not writing */
     STA(outbufrem) = csound->oparms->outbufsamps;
 }
