@@ -252,11 +252,9 @@ int AuHAL_open(CSOUND *csound, const csRtAudioParams * parm,
       csound->Free(csound,b);
     }
 
-
-    if(cdata->disp)
-      csound->Message(csound,
-         "==========================================================\n");
-
+     OPARMS O;
+    csound->GetOParms(csound, &O);
+    if(O.msglevel || O.odebug) {
     if (isInput)
       csound->Message(csound,
                       Str("AuHAL Module: found %d input device(s):\n"), devins);
@@ -279,6 +277,7 @@ int AuHAL_open(CSOUND *csound, const csRtAudioParams * parm,
                           devinfo[i].outchannels);
       }
     }
+      }
 
     if (parm->devName != NULL) devnum = atoi(parm->devName);
     else devnum = parm->devNum;
@@ -288,7 +287,10 @@ int AuHAL_open(CSOUND *csound, const csRtAudioParams * parm,
       prop.mSelector = kAudioHardwarePropertyDevices;
       if (isInput) {
         for(i=0; (unsigned int)  i < devnos; i++) {
-          if((unsigned int) devinfo[i].indevnum == devnum) CoreAudioDev = i;
+          if((unsigned int) devinfo[i].indevnum == devnum) {
+            CoreAudioDev = i;
+            break;
+          }
         }
         if (LIKELY(CoreAudioDev >= 0)) {
           prop.mSelector = kAudioHardwarePropertyDefaultInputDevice;
@@ -296,8 +298,11 @@ int AuHAL_open(CSOUND *csound, const csRtAudioParams * parm,
           AudioObjectSetPropertyData(kAudioObjectSystemObject, &prop,
                                      0, NULL, sizeof(AudioDeviceID), &dev);
         }
-        else csound->Warning(csound, Str("requested device %d out of range"),
+        else {
+       if(O.msglevel || O.odebug)
+          csound->Warning(csound, Str("requested device %d out of range"),
                              devnum);
+        }
 
       }
       else {
@@ -311,18 +316,40 @@ int AuHAL_open(CSOUND *csound, const csRtAudioParams * parm,
                                      0, NULL, sizeof(AudioDeviceID), &dev);
 
         }
-        else csound->Warning(csound, Str("requested device %d (%s) out of range"),
+        else {
+          if(O.msglevel || O.odebug)
+          csound->Warning(csound, Str("requested device %d (%s) out of range"),
                              devnum, devinfo[CoreAudioDev].name);
+        }
       }
     }
 
-    for(i=0; (unsigned int)  i < devnos; i++)
-        if(sysdevs[i] == dev){
-          if(isInput) ADC_channels(csound, devinfo[i].inchannels);
-          else DAC_channels(csound, devinfo[i].outchannels);
+    for(i=0; (unsigned int)  i < devnos; i++) {
+      if(sysdevs[i] == dev){
+        if(isInput) {
+          if(devinfo[i].inchannels < parm->nChannels) {
+            if(O.msglevel || O.odebug)
+            csound->ErrorMsg(csound,
+                             Str(" *** CoreAudio: Device has not enough"
+                                 " inputs (%d, requested %d)\n"),
+                             devinfo[i].inchannels, parm->nChannels);
+            return -1;
+          }
+          ADC_channels(csound, devinfo[i].inchannels);
         }
-
-
+        else {
+          if(devinfo[i].outchannels < parm->nChannels) {
+            if(O.msglevel || O.odebug)
+            csound->ErrorMsg(csound,
+                             Str(" *** CoreAudio: Device has not enough"
+                                 " outputs (%d, requested %d)\n"),
+                             devinfo[i].outchannels, parm->nChannels);
+            return -1;
+          }
+          DAC_channels(csound, devinfo[i].outchannels);
+        }
+      }
+    }
     csound->Free(csound,sysdevs);
     csound->Free(csound,devinfo);
 
@@ -330,6 +357,7 @@ int AuHAL_open(CSOUND *csound, const csRtAudioParams * parm,
     prop.mSelector = kAudioObjectPropertyName;
     AudioObjectGetPropertyData(dev,
                                &prop, 0, NULL, &psize, &devName);
+    if(O.msglevel || O.odebug) {
     if(isInput)
       csound->Message(csound, Str("selected input device: %s\n"),
                       CFStringGetCStringPtr(devName, defaultEncoding));
@@ -337,7 +365,7 @@ int AuHAL_open(CSOUND *csound, const csRtAudioParams * parm,
     else
       csound->Message(csound, Str("selected output device: %s\n"),
                       CFStringGetCStringPtr(devName, defaultEncoding));
-
+    }
     CFRelease(devName);
 
     srate = csound->GetSr(csound);
@@ -366,7 +394,8 @@ int AuHAL_open(CSOUND *csound, const csRtAudioParams * parm,
     if(srate < 0)
       srate  =  csound->system_sr(csound, sr);
     if(UNLIKELY(sr != srate)) {
-      csound->Warning(csound,
+      if(O.msglevel || O.odebug)
+       csound->Warning(csound,
                       Str("Attempted to set device SR, tried %.1f, got %.1f\n"),
                       srate, sr);
     }
@@ -442,7 +471,8 @@ int AuHAL_open(CSOUND *csound, const csRtAudioParams * parm,
       AudioUnitInitialize(*aunit);
       AudioOutputUnitStart(*aunit);
 
-      csound->Message(csound,
+      if(O.msglevel || O.odebug)
+       csound->Message(csound,
                       Str("***** AuHAL module: output device open with %d "
                           "buffer frames\n"),
                           bufframes);
@@ -469,14 +499,12 @@ int AuHAL_open(CSOUND *csound, const csRtAudioParams * parm,
                            kAudioUnitScope_Input, isInput, &input, sizeof(input));
       AudioUnitInitialize(*aunit);
       AudioOutputUnitStart(*aunit);
-      csound->Message(csound,
+      if(O.msglevel || O.odebug)
+       csound->Message(csound,
                       Str("***** AuHAL module: input device open with "
                           "%d buffer frames\n"),
                       (int) bufframes);
     }
-    if(!cdata->disp)
-      csound->Message(csound,
-              "==========================================================\n");
 
     cdata->disp = 0;
     return 0;
@@ -627,6 +655,8 @@ static int recopen_(CSOUND *csound, const csRtAudioParams * parm)
     cdata->incb =
       csound->CreateCircularBuffer(csound,
                                    parm->bufSamp_HW*parm->nChannels, sizeof(MYFLT));
+
+
     int ret = AuHAL_open(csound, parm, cdata, 1);
     return ret;
 }
@@ -655,6 +685,7 @@ static int playopen_(CSOUND *csound, const csRtAudioParams * parm)
     cdata->outcb =
       csound->CreateCircularBuffer(csound,
                                    parm->bufSamp_HW*parm->nChannels, sizeof(MYFLT));
+
     return AuHAL_open(csound, parm,cdata,0);
 }
 
@@ -816,13 +847,18 @@ static void rtclose_(CSOUND *csound)
       csound->DestroyCircularBuffer(csound, cdata->incb);
       csound->DestroyCircularBuffer(csound, cdata->outcb);
       csound->Free(csound,cdata);
-      csound->Message(csound, "%s", Str("AuHAL module: device closed\n"));
+      OPARMS O;
+      csound->GetOParms(csound, &O);
+      if(O.msglevel || O.odebug)
+       csound->Message(csound, "%s", Str("AuHAL module: device closed\n"));
     }
 }
 
 int csoundModuleInit(CSOUND *csound)
 {
     char   *drv;
+    OPARMS O;
+    csound->GetOParms(csound, &O);
     csound->module_list_add(csound, "auhal", "audio");
     drv = (char *) csound->QueryGlobalVariable(csound, "_RTAUDIO");
     if (drv == NULL)
@@ -832,7 +868,7 @@ int csoundModuleInit(CSOUND *csound)
           strcmp(drv, "coreaudio") == 0 || strcmp(drv, "CoreAudio") == 0 ||
           strcmp(drv, "COREAUDIO") == 0))
       return 0;
-    //if (csound->oparms->msglevel & 0x400)
+   if(O.msglevel || O.odebug)
     csound->Message(csound, "%s", Str("rtaudio: coreaaudio-AuHAL module enabled\n"));
     csound->SetPlayopenCallback(csound, playopen_);
     csound->SetRecopenCallback(csound, recopen_);
