@@ -45,26 +45,27 @@ void csound_aops_init_tables(CSOUND *csound)
     int32_t   i;
     if (csound->cpsocfrc==NULL)
       csound->cpsocfrc = (MYFLT *) csound->Malloc(csound, sizeof(MYFLT)*OCTRES);
-    if (csound->powerof2==NULL)
-      csound->powerof2 = (MYFLT *) csound->Malloc(csound,
-                                                  sizeof(MYFLT)*POW2TABSIZI);
+    /* if (csound->powerof2==NULL) */
+    /*   csound->powerof2 = (MYFLT *) csound->Malloc(csound, */
+    /*                                               sizeof(MYFLT)*POW2TABSIZI); */
     for (i = 0; i < OCTRES; i++)
       csound->cpsocfrc[i] = POWER(FL(2.0), (MYFLT)i / OCTRES) * ONEPT;
-    for (i = 0; i < POW2TABSIZI; i++) {
-      csound->powerof2[i] =
-        POWER(FL(2.0), (MYFLT)i * (MYFLT)(1.0/POW2TABSIZI) - FL(POW2MAX));
-    }
+    /* for (i = 0; i < POW2TABSIZI; i++) { */
+    /*   csound->powerof2[i] = */
+    /*     POWER(FL(2.0), (MYFLT)i * (MYFLT)(1.0/POW2TABSIZI) - FL(POW2MAX)); */
+    /* } */
 }
 
 
 MYFLT csoundPow2(CSOUND *csound, MYFLT a)
 {
-    int32_t n;
+    /* int32_t n; */
     if (a > POW2MAX) a = POW2MAX;
     else if (a < -POW2MAX) a = -POW2MAX;
+    return POWER(FL(2.0), a);
     /* 4096 * 15 */
-    n = (int32_t)MYFLT2LRND(a * FL(POW2TABSIZI)) + POW2MAX*POW2TABSIZI;
-    return ((MYFLT) (1UL << (n >> 12)) * csound->powerof2[n & (POW2TABSIZI-1)]);
+    /* n = (int32_t)MYFLT2LRND(a * FL(POW2TABSIZI)) + POW2MAX*POW2TABSIZI; */
+    /* return ((MYFLT) (1UL << (n >> 12)) * csound->powerof2[n & (POW2TABSIZI-1)]); */
 }
 
 
@@ -481,10 +482,45 @@ AA_VEC(divaa,_mm_div_pd)
 AA(addaa,+)
 AA(subaa,-)
 AA(mulaa,*)
-AA(divaa,/)
+//AA(divaa,/)
 #endif
 
-int32_t modaa(CSOUND *csound, AOP *p)
+int32_t divaa(CSOUND *csound, AOP *p)
+{
+    MYFLT   *r, *a, *b;
+    int     err = 0;
+    IGN(csound);
+    uint32_t n, nsmps = CS_KSMPS;
+    if (LIKELY(nsmps!=1)) {
+      uint32_t offset = p->h.insdshead->ksmps_offset;
+      uint32_t early  = p->h.insdshead->ksmps_no_end;
+      r = p->r;
+      a = p->a;
+      b = p->b;
+      if (UNLIKELY(offset)) memset(r, '\0', offset*sizeof(MYFLT));
+      if (UNLIKELY(early)) {
+        nsmps -= early;
+        memset(&r[nsmps], '\0', early*sizeof(MYFLT));
+      }
+      for (n=offset; n<nsmps; n++ ) {
+        MYFLT bb = b[n];
+        if (UNLIKELY(bb==FL(0.0) && err==0)) {
+          csound->Warning(csound, Str("Division by zero"));
+          err = 1;
+        }
+        r[n] = a[n] / bb;
+      }
+      return OK;
+    }
+    else {
+      if (UNLIKELY(*p->b==FL(0.0)))
+        csound->Warning(csound, Str("Division by zero"));
+      *p->r = *p->a / *p->b;
+      return OK;
+    }
+}
+
+  int32_t modaa(CSOUND *csound, AOP *p)
 {
     MYFLT   *r, *a, *b;
     IGN(csound);
@@ -530,7 +566,7 @@ int32_t divzka(CSOUND *csound, DIVZ *p)
       nsmps -= early;
       memset(&r[nsmps], '\0', early*sizeof(MYFLT));
     }
-    for (n=offset; n<nsmps; n++) {
+     for (n=offset; n<nsmps; n++) {
       MYFLT bb = b[n];
       r[n] = (bb==FL(0.0) ? def : a / bb);
     }
@@ -763,6 +799,13 @@ int32_t int1a_ceil(CSOUND *csound, EVAL *p)         /* round up */
 
 #define rndmlt (105.947)
 
+int32_t rnd1seed(CSOUND *csound, INM *p)
+{
+    double intpart;
+    csound->rndfrac = modf(*p->ar, &intpart);
+    return OK;
+}
+
 int32_t rnd1(CSOUND *csound, EVAL *p)               /* returns unipolar rand(x) */
 {
     double intpart;
@@ -1081,9 +1124,11 @@ int32_t cpspch(CSOUND *csound, EVAL *p)
 
 int32_t cpsmidinn(CSOUND *csound, EVAL *p)
 {
-    IGN(csound);
-    *p->r = pow(FL(2.0),
-                (*p->a - FL(69.0)) / FL(12.0)) * (MYFLT)(csound->A4);
+    MYFLT note = *p->a;         /* (note-69)>12* */
+    if (note > 12*32+69 || note < 0)
+      return csound->InitError(csound, Str("MIDI note %f out of range"), note);
+    *p->r = POWER(FL(2.0),
+                  (note - FL(69.0)) / FL(12.0)) * (MYFLT)(csound->A4);
     return OK;
 }
 
@@ -1121,16 +1166,16 @@ int32_t cpsxpch(CSOUND *csound, XENH *p)
     else {                      /* Values in a table */
       MYFLT t = - *p->et;
       FUNC* ftp = csound->FTnp2Finde(csound, &t);
-      int32_t len;
+      int32_t len, frt;
       if (UNLIKELY(ftp == NULL))
         return csound->PerfError(csound, &(p->h),Str("No tuning table %d"),
                                  -((int32_t)*p->et));
       len = ftp->flen;
-      while (fract>len) {
-        fract -= len; loct++;
+      frt = (int32_t)(100.0*fract+0.5);
+      while (frt>len) {
+        frt -= len; loct++;
       }
-      fract += 0.005;
-      *p->r = *p->ref * *(ftp->ftable + (int32_t)(100.0*fract)) *
+      *p->r = *p->ref * *(ftp->ftable + frt) *
         POWER(*p->cy, (MYFLT)loct);
     }
     return OK;
@@ -1149,16 +1194,18 @@ int32_t cps2pch(CSOUND *csound, XENH *p)
     else {
       MYFLT t = - *p->et;
       FUNC* ftp = csound->FTnp2Finde(csound, &t);
-      int32_t len;
+      int32_t len, frt;
       if (UNLIKELY(ftp == NULL))
         return csound->PerfError(csound, &(p->h),Str("No tuning table %d"),
                                  -((int32_t)*p->et));
       len = ftp->flen;
-      while (fract>len) {
-        fract -= len; loct++;
+      frt = (int32_t)(100.0*fract+0.5);
+      //printf("len=%d fract=%g frt=%d\n", len, fract, frt);
+      while (frt>len) {
+        frt -= len; loct++;
       }
-      fract += 0.005;
-      *p->r = (MYFLT)(1.02197503906 * *(ftp->ftable +(int32_t)(100.0*fract)) *
+      //printf("len=%d loct=%g frt=%d\n", len, loct, frt);
+      *p->r = (MYFLT)(1.02197503906 * *(ftp->ftable + frt) *
                       pow(2.0, loct));
     }
 
@@ -1255,7 +1302,7 @@ int32_t logbasetwo_set(CSOUND *csound, EVAL *p)
 
 int32_t powoftwo(CSOUND *csound, EVAL *p)
 {
-    *p->r = csound->Pow2(csound,*p->a);
+    *p->r = POWER(FL(2.0), *p->a);
     return OK;
 }
 
@@ -1271,7 +1318,7 @@ int32_t powoftwoa(CSOUND *csound, EVAL *p)
       memset(&r[nsmps], '\0', early*sizeof(MYFLT));
     }
     for (n = offset; n < nsmps; n++)
-      r[n] = csound->Pow2(csound,a[n]);
+      r[n] = POWER(FL(2.0), a[n]);
     return OK;
 }
 
@@ -1281,7 +1328,7 @@ int32_t powoftwoa(CSOUND *csound, EVAL *p)
 int32_t semitone(CSOUND *csound, EVAL *p)
 {
     MYFLT a = *p->a*ONEd12;
-    *p->r = csound->Pow2(csound,a);
+    *p->r = POWER(FL(2.0), a);
     return OK;
 }
 
@@ -1300,15 +1347,15 @@ int32_t asemitone(CSOUND *csound, EVAL *p)            /* JPff */
     }
     for (n = offset; n < nsmps; n++) {
       MYFLT aa = (a[n])*ONEd12;
-      r[n] = csound->Pow2(csound,aa);
+      r[n] = POWER(FL(2.0), aa);
     }
     return OK;
 }
 
 int32_t cent(CSOUND *csound, EVAL *p)
 {
-    MYFLT a = *p->a*ONEd1200;
-    *p->r = csound->Pow2(csound,a);
+    MYFLT a = *p->a;
+    *p->r = POWER(FL(2.0), a/FL(1200.0));
     return OK;
 }
 
@@ -1327,7 +1374,7 @@ int32_t acent(CSOUND *csound, EVAL *p)        /* JPff */
     }
     for (n = offset; n < nsmps; n++) {
       MYFLT aa = (a[n])*ONEd1200;
-      r[n] = csound->Pow2(csound,aa);
+      r[n] = POWER(FL(2.0), aa);
   }
   return OK;
 }
@@ -1336,7 +1383,7 @@ int32_t acent(CSOUND *csound, EVAL *p)        /* JPff */
 
 int32_t db(CSOUND *csound, EVAL *p)
 {
-    *p->r = csound->Pow2(csound,*p->a*LOG2_10D20);
+    *p->r = POWER(FL(2.0), *p->a*LOG2_10D20);
     return OK;
 }
 
@@ -1355,7 +1402,7 @@ int32_t dba(CSOUND *csound, EVAL *p)          /* JPff */
     }
     for (n = offset; n < nsmps; n++) {
       MYFLT aa = a[n];
-      r[n] = csound->Pow2(csound,aa*LOG2_10D20);
+      r[n] = POWER(FL(2.0), aa*LOG2_10D20);
     }
     return OK;
 }
@@ -1655,7 +1702,7 @@ int32_t inch_opcode1(CSOUND *csound, INCH1 *p)
     ch = MYFLT2LRND(*p->ch);
     if (UNLIKELY(ch > (uint32_t)csound->inchnls)) {
       if (p->init)
-        csound->Message(csound, Str("Input channel %d too large; ignored"), ch);
+        csound->Message(csound, Str("Input channel %d too large; ignored\n"), ch);
       memset(p->ar, 0, sizeof(MYFLT)*nsmps);
       p->init = 0;
       //        return OK;
@@ -1880,17 +1927,22 @@ int32_t outq4(CSOUND *csound, OUTM *p)
 
 inline static int32_t outn(CSOUND *csound, uint32_t n, OUTX *p)
 {
-    uint32_t nsmps =CS_KSMPS,  i, j, k=0;
+    uint32_t nsmps = CS_KSMPS,  i, j, k=0;
     MYFLT *spout = CS_SPOUT; ///csound->spraw;
     uint32_t offset = p->h.insdshead->ksmps_offset;
     uint32_t early  = p->h.insdshead->ksmps_no_end;
-    //    if (UNLIKELY((offset|early))) {
+    //        if (UNLIKELY((offset|early))) {
+    //          printf("OUT; spout=%p early=%d offset=%d\n", spout, early, offset);}
     early = nsmps - early;
     CSOUND_SPOUT_SPINLOCK
 
     if (!csound->spoutactive) {
+      //printf("inactive: spout=%p size=%ld\n",
+      //       spout, csound->nspout*sizeof(MYFLT));
       memset(spout, '\0', csound->nspout*sizeof(MYFLT));
       for (i=0; i<n; i++) {
+        //printf("        : spout=%p size=%ld\n",
+        //       spout+k+offset, (early-offset)*sizeof(MYFLT));
         memcpy(&spout[k+offset], p->asig[i]+offset, (early-offset)*sizeof(MYFLT));
         k += nsmps;
       }
@@ -1899,6 +1951,7 @@ inline static int32_t outn(CSOUND *csound, uint32_t n, OUTX *p)
     else {
       for (i=0; i<n; i++) {
         for (j=offset; j<early; j++) {
+          //printf("active: spout=%p k=%d j=%d\n", spout, k, j);
           spout[k + j] += p->asig[i][j];
         }
         k += nsmps;
@@ -2050,6 +2103,43 @@ int32_t outch(CSOUND *csound, OUTCH *p)
     return OK;
 }
 
+int32_t outrep(CSOUND *csound, OUTM *p)
+{
+    uint32_t nsmps = CS_KSMPS,  i, j, k=0;
+    uint32_t n = csound->nchnls;
+    MYFLT *spout = CS_SPOUT; ///csound->spraw;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    //if (UNLIKELY((offset|early))) {
+    //  printf("OUT; spout=%p early=%d offset=%d\n", spout, early, offset);}
+    early = nsmps - early;
+    CSOUND_SPOUT_SPINLOCK
+
+    if (!csound->spoutactive) {
+      //printf("inactive: spout=%p size=%ld\n",
+      //       spout, csound->nspout*sizeof(MYFLT));
+      memset(spout, '\0', csound->nspout*sizeof(MYFLT));
+      for (i=0; i<n; i++) {
+        //printf("        : spout=%p size=%ld\n",
+        //       spout+k+offset, (early-offset)*sizeof(MYFLT));
+        memcpy(&spout[k+offset], p->asig+offset, (early-offset)*sizeof(MYFLT));
+        k += nsmps;
+      }
+      csound->spoutactive = 1;
+    }
+    else {
+      for (i=0; i<n; i++) {
+        for (j=offset; j<early; j++) {
+          //printf("active: spout=%p k=%d j=%d\n", spout, k, j);
+          spout[k + j] += p->asig[j];
+        }
+        k += nsmps;
+      }
+    }
+    CSOUND_SPOUT_SPINUNLOCK
+    return OK;
+}
+
 /* For parallel mixin template */
 int32_t addina(CSOUND *csound, ASSIGN *p)
 {
