@@ -141,7 +141,7 @@ int32_t kdsplay(CSOUND *csound, DSPLAY *p)
         p->pntcnt = p->npts;
         if (fp >= p->endp) {
           fp = p->begp;
-          fp2 = fp + p->bufpts;
+          fp2 = fp + p->bufpts; /* FIXME: Unused */
         }
         p->dwindow.fdata = fp;  /* display from fp */
         display(csound, &p->dwindow);
@@ -246,10 +246,11 @@ int32_t fftset(CSOUND *csound, DSPFFT *p) /* fftset, dspfft -- calc Fast Fourier
 
     window_size = (int32_t)*p->inpts;
     if (UNLIKELY(window_size > WINDMAX)) {
-      return csound->InitError(csound, Str("too many points requested"));
+      return csound->InitError(csound, Str("too many points requested (%d)"), window_size);
     }
     if (UNLIKELY(window_size < WINDMIN)) {
-      return csound->InitError(csound, Str("too few points requested"));
+      return csound->InitError(csound, Str("too few points requested (%d), minimum is %d"),
+                               window_size, WINDMIN);
     }
     if (UNLIKELY(window_size < 1L || (window_size & (window_size - 1L)) != 0L)) {
       return csound->InitError(csound, Str("window size must be power of two"));
@@ -383,7 +384,7 @@ int32_t kdspfft(CSOUND *csound, DSPFFT *p)
         MYFLT *hWin = (MYFLT *) p->auxch.auxp;
         d_fft(csound, p->sampbuf, csound->disprep_fftcoefs,
               p->windsize, hWin, p->dbout, p->overN);
-        tp = csound->disprep_fftcoefs;
+        //tp = csound->disprep_fftcoefs; UNUSED
         //tplim = tp + p->ncoefs;
         //do {
         // *tp *= p->overN;            /* scale 1/N */
@@ -433,7 +434,7 @@ int32_t dspfft(CSOUND *csound, DSPFFT *p)
           MYFLT *hWin = (MYFLT *) p->auxch.auxp;
           d_fft(csound, p->sampbuf, csound->disprep_fftcoefs,
                 p->windsize, hWin, p->dbout, p->overN);
-          tp = csound->disprep_fftcoefs;
+          //tp = csound->disprep_fftcoefs; UNUSED
           //tplim = tp + p->ncoefs;
           //do {
           //  *tp *= p->overN;              /* scale 1/N */
