@@ -166,7 +166,7 @@ int32_t printk(CSOUND *csound, PRINTK *p)
     if (UNLIKELY(p->initialised != -1))
       csound->PerfError(csound, &(p->h), Str("printk not initialised"));
 
-    //printf("printk: KCNT = %lu\n", CS_KCNT);
+    //printf("printk: KCNT = %llu\n", CS_KCNT);
     //printf("printat = %lf\n", p->printat);
     /* Now test if the cycle number has reached the next print time */
     if (p->printat <= CS_KCNT-1) {
@@ -544,7 +544,7 @@ int32_t printks(CSOUND *csound, PRINTKS *p)
     if (p->printat <= CS_KCNT-1) {
       //string[0]='\0';           /* incase of empty string */
       memset(string,0,8192);
-      if (sprints(string, p->txtstring, p->kvals, p->INOCOUNT-2)==NOTOK)
+      if (sprints(string, p->txtstring, p->kvals, p->INOCOUNT-2)!=OK)
         return
           csound->PerfError(csound,  &(p->h),
                             Str("Insufficient arguments in formatted printing"));
@@ -566,7 +566,8 @@ int32_t printsset(CSOUND *csound, PRINTS *p)
     pk.ptime = &ptime;
     printksset(csound, &pk);
     memset(string,0,8192);
-    if (sprints(string, pk.txtstring, p->kvals, p->INOCOUNT-1)==NOTOK)
+    memset(pk.txtstring,0,sizeof(pk.txtstring));
+    if (sprints(string, pk.txtstring, p->kvals, p->INOCOUNT-1)!=OK)
         return
           csound->InitError(csound,
                             Str("Insufficient arguments in formatted printing"));
@@ -586,7 +587,7 @@ int32_t printsset_S(CSOUND *csound, PRINTS *p)
     printksset_S(csound, &pk);
     if (strlen(pk.txtstring) < 8191){
       memset(string,0,8192);
-    if (sprints(string, pk.txtstring, p->kvals, p->INOCOUNT-1)==NOTOK)
+    if (sprints(string, pk.txtstring, p->kvals, p->INOCOUNT-1)!=OK)
         return
           csound->InitError(csound,
                             Str("Insufficient arguments in formatted printing"));
@@ -699,7 +700,7 @@ int32_t printk3(CSOUND *csound, PRINTK3 *p)
       MYFLT *vv[1];
       vv[0] = &value;
       buff[0] = '\0';
-      if (sprints(buff, p->sarg, vv, 1)==NOTOK)
+      if (sprints(buff, p->sarg, vv, 1)!=OK)
         return
           csound->PerfError(csound,  &(p->h),
                             Str("Insufficient arguments in formatted printing"));
