@@ -22,6 +22,9 @@
     02110-1301 USA
 */
 
+/* Haiku 'int32' etc definitions in net headers conflict with sysdep.h */
+#define __HAIKU_CONFLICT
+
 #include "csdl.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -584,7 +587,7 @@ static int32_t OSC_deinit(CSOUND *csound, OSCINIT *p)
     lo_server_thread_stop(ports[n].thread);
     lo_server_thread_free(ports[n].thread);
     ports[n].thread =  NULL;
-    csound->Message(csound, "%s", Str("OSC deinitiatised\n"));
+    csound->Message(csound, "%s", Str("OSC deinitialised\n"));
     return OK;
 }
 
