@@ -1,12 +1,4 @@
-
 /*
-    ftgen.c:
-
-    Copyright (C) 1991, 1994, 1995, 1998, 2000, 2004
-                  Barry Vercoe, John ffitch, Paris Smaragdis,
-                  Gabriel Maldonado, Richard Karpen, Greg Sullivan,
-                  Pete Moss, Istvan Varga, Victor Lazzarini
-
     This file is part of Csound.
 
     The Csound Library is free software; you can redistribute it
@@ -103,43 +95,43 @@ static int32_t ftgen_(CSOUND *csound, FTGEN *p, int32_t istring1, int32_t istrin
     fp[4] = *p->p4;
 
 
-      if (istring1) {              /* Named gen */
-        NAMEDGEN *named = (NAMEDGEN*) csound->GetNamedGens(csound);
-        while (named) {
-          if (strcmp(named->name, ((STRINGDAT *) p->p4)->data) == 0) {
-            /* Look up by name */
-            fp[4] = named->genum;
-            break;
-          }
-          named = named->next;                            /*  and round again   */
-        }
-        if (UNLIKELY(named == NULL)) {
-          csound->Free(csound,ftevt);
-          return csound->InitError(csound,
-                                   Str("Named gen \"%s\" not defined"),
-                                   (char *)p->p4);
-        }
-        else fp[4] = named->genum;
-      }
-
-      if (istring2) {  /* string argument: */
-        n = (int32_t) fp[4];
-        fp[5] = SSTRCOD;
-        if (n < 0)
-          n = -n;
-        switch (n) {                      /*   must be Gen01, 23, 28, 43, 49 */
-        case 1:
-        case 23:
-        case 28:
-        case 43:
-        case 49:
-          ftevt->strarg = ((STRINGDAT *) p->p5)->data;
+    if (istring1) {              /* Named gen */
+      NAMEDGEN *named = (NAMEDGEN*) csound->GetNamedGens(csound);
+      while (named) {
+        if (strcmp(named->name, ((STRINGDAT *) p->p4)->data) == 0) {
+          /* Look up by name */
+          fp[4] = named->genum;
           break;
-        default:
-          csound->Free(csound, ftevt);
-          return csound->InitError(csound, Str("ftgen string arg not allowed"));
         }
+        named = named->next;                            /*  and round again   */
       }
+      if (UNLIKELY(named == NULL)) {
+        csound->Free(csound,ftevt);
+        return csound->InitError(csound,
+                                 Str("Named gen \"%s\" not defined"),
+                                 (char *)p->p4);
+      }
+      // else fp[4] = named->genum;
+    }
+
+    if (istring2) {  /* string argument: */
+      n = (int32_t) fp[4];
+      fp[5] = SSTRCOD;
+      if (n < 0)
+        n = -n;
+      switch (n) {                      /*   must be Gen01, 23, 28, 43, 49 */
+      case 1:
+      case 23:
+      case 28:
+      case 43:
+      case 49:
+        ftevt->strarg = ((STRINGDAT *) p->p5)->data;
+        break;
+      default:
+        csound->Free(csound, ftevt);
+        return csound->InitError(csound, Str("ftgen string arg not allowed"));
+      }
+    }
     else {
       fp[5] = *p->p5;                                   /* else no string */
     }
@@ -421,7 +413,7 @@ static int32_t ftload_(CSOUND *csound, FTLOAD *p, int32_t istring)
         header.gen01args.sample_rate = (MYFLT)cs_strtod(s1, &endptr);
         if (UNLIKELY(endptr==NULL)) goto err4;
         if (UNLIKELY(NULL==fgets(s, 64, file))) {goto err4;}
-        s1 = strchr(s, ' ')+1;
+        //s1 = strchr(s, ' ')+1;
         /* WARNING! skips header.gen01args.strarg from saving/loading
            in text format */
         header.fno = (int32) fno;
@@ -660,39 +652,34 @@ static int32_t ftgen_list(CSOUND *csound, FTGEN *p, int32_t istring)
     fp[4] = *p->p4;
 
 
-      if (istring) {              /* Named gen */
-        NAMEDGEN *named = (NAMEDGEN*) csound->GetNamedGens(csound);
-        while (named) {
-          if (strcmp(named->name, ((STRINGDAT *) p->p4)->data) == 0) {
-            /* Look up by name */
-            fp[4] = named->genum;
-           break;
-          }
-          named = named->next;                            /*  and round again   */
+    if (istring) {              /* Named gen */
+      NAMEDGEN *named = (NAMEDGEN*) csound->GetNamedGens(csound);
+      while (named) {
+        if (strcmp(named->name, ((STRINGDAT *) p->p4)->data) == 0) {
+          /* Look up by name */
+          fp[4] = named->genum;
+          break;
         }
-        if (UNLIKELY(named == NULL)) {
-          csound->Free(csound,ftevt);
-          return csound->InitError(csound,
-                                   Str("Named gen \"%s\" not defined"),
-                                   (char *)p->p4);
-        }
-        else fp[4] = named->genum;
+        named = named->next;                            /*  and round again   */
       }
+      if (UNLIKELY(named == NULL)) {
+        csound->Free(csound,ftevt);
+        return csound->InitError(csound,
+                                 Str("Named gen \"%s\" not defined"),
+                                 (char *)p->p4);
+      }
+    }
 
-      ARRAYDAT *array = (ARRAYDAT*) (p->p5);
-      n = array->sizes[0];
-      ftevt->pcnt = (int16) n+4;
-      int32_t i = 0;
-      memcpy(&fp[5], &array->data[0], n*sizeof(MYFLT));
-      while (i < n) {
-        i++;
-      }
-      n = csound->hfgens(csound, &ftp, ftevt, 1);         /* call the fgen */
-     csound->Free(csound,ftevt);
-     if (UNLIKELY(n != 0))
-       return csound->InitError(csound, Str("ftgen error"));
-     if (ftp != NULL)
-       *p->ifno = (MYFLT) ftp->fno;                      /* record the fno */
+    ARRAYDAT *array = (ARRAYDAT*) (p->p5);
+    n = array->sizes[0];
+    ftevt->pcnt = (int16) n+4;
+    memcpy(&fp[5], array->data, n*sizeof(MYFLT));
+    n = csound->hfgens(csound, &ftp, ftevt, 1);         /* call the fgen */
+    csound->Free(csound,ftevt);
+    if (UNLIKELY(n != 0))
+      return csound->InitError(csound, Str("ftgen error"));
+    if (ftp != NULL)
+      *p->ifno = (MYFLT) ftp->fno;                      /* record the fno */
     return OK;
 }
 
@@ -704,6 +691,86 @@ static int32_t ftgen_list_i(CSOUND *csound, FTGEN *p){
     return ftgen_list(csound,p,0);
 }
 
+/*
+ gtftargs (c) 2016 Guillermo Senna.
+*/
+
+typedef struct {
+   OPDS      h;
+   STRINGDAT *Scd;
+   MYFLT     *ftable;
+   MYFLT     *ktrig;
+   MYFLT     prv_ktrig;
+   int32_t       status;
+} FTARGS;
+
+static int32_t getftargs(CSOUND *, FTARGS *);
+
+/*
+    Inspiration for the implementation of this Opcode was taken
+    from the following Csound Opcodes: "sprintf", "puts" and "pwd".
+    Credit for that goes to their respective authors.
+*/
+
+static int32_t getftargs_init(CSOUND *csound, FTARGS *p)
+{
+    p->status = OK;
+    if (*p->ktrig > FL(0.0))
+      p->status = getftargs(csound, p);
+    p->prv_ktrig = *p->ktrig;
+
+    return p->status;
+}
+
+static int32_t getftargs_process(CSOUND *csound, FTARGS *p)
+{
+   if (*p->ktrig != p->prv_ktrig && *p->ktrig > FL(0.0)) {
+     p->prv_ktrig = *p->ktrig;
+     p->status = getftargs(csound, p);
+   }
+
+   return p->status;
+
+}
+
+
+static int32_t getftargs(CSOUND *csound, FTARGS *p)
+{
+    FUNC *src;
+    int32 argcnt, i, strlen = 0;
+
+    if (UNLIKELY((src = csound->FTnp2Find(csound, p->ftable)) == NULL)) {
+      return csound->PerfError(csound, &(p->h),
+                               Str("table: could not find ftable %d"),
+                               (int32_t) *p->ftable);
+    }
+
+    argcnt = src->argcnt;
+
+    for (i = 1; i != argcnt; i++)
+      strlen += snprintf(NULL, 0, "%g ", src->args[i]);
+
+    p->Scd->size = strlen;
+
+    if (p->Scd->data == NULL) {
+      p->Scd->data = (char*) csound->Calloc(csound, strlen);
+    }
+    else
+      p->Scd->data = (char*) csound->ReAlloc(csound, p->Scd->data, strlen);
+
+    {
+      char* curr = p->Scd->data, *const end = curr + strlen;
+      for (i = 1; curr != end && i != argcnt; i++) {
+        curr += snprintf(curr, end-curr, "%g ", src->args[i]);
+      }
+    }
+
+    return OK;
+}
+
+
+
+
 #define S(x)    sizeof(x)
 
 static OENTRY localops[] = {
@@ -729,7 +796,10 @@ static OENTRY localops[] = {
   { "ftloadk",  S(FTLOAD_K),  TW, 3,  "",   "ikim",   (SUBR) ftsave_k_set,
     (SUBR) ftload_k, NULL       },
   { "ftloadk.S",  S(FTLOAD_K),  TW, 3,  "",   "Skim",   (SUBR) ftsave_k_set,
-                                                  (SUBR) ftload_kS, NULL       }
+    (SUBR) ftload_kS, NULL       },
+    { "getftargs",   sizeof(FTARGS),  0, 3, "S", "ik",
+    (SUBR)getftargs_init, (SUBR)getftargs_process }
+
 };
 
 int32_t ftgen_init_(CSOUND *csound)
