@@ -304,14 +304,13 @@ int32_t mp3len_(CSOUND *csound, MP3LEN *p, int32_t stringname)
       return csound->InitError(csound, "%s", mp3dec_error(r));
     }
     close(fd);
-
-    if(!strcmp(csound->GetOpcodeName(&p->h), "mp3len"))
+    if(!strcmp(csound->GetOpcodeName(&p->h), "mp3len.i"))
       *p->ir = (MYFLT)mpainfo.duration;
-    else if(!strcmp(csound->GetOpcodeName(&p->h), "mp3sr"))
+    else if(!strcmp(csound->GetOpcodeName(&p->h), "mp3sr.i"))
       *p->ir = (MYFLT) mpainfo.frequency;
-    else if(!strcmp(csound->GetOpcodeName(&p->h), "mp3bitrate"))
+    else if(!strcmp(csound->GetOpcodeName(&p->h), "mp3bitrate.i"))
       *p->ir = (MYFLT) mpainfo.bitrate;
-    else if(!strcmp(csound->GetOpcodeName(&p->h), "mp3nchnls"))
+    else if(!strcmp(csound->GetOpcodeName(&p->h), "mp3nchnls.i"))
       *p->ir = (MYFLT) mpainfo.channels;
 
     mp3dec_uninit(mpa);
@@ -530,12 +529,13 @@ static int32_t sinit3_(CSOUND *csound, DATASPACE *p)
       fdrecord(csound, &(p->fdch));
     */
     printf("fftsize = %d\n", p->N);
-    int32_t buffersize = size;
-    buffersize /= mpainfo.decoded_sample_size;
     int32_t skip = (int32_t)(*p->skip*CS_ESR)*p->resamp;
     p->bufused = -1;
 
-    /*while (skip > 0) {
+    /*
+    int32_t buffersize = size;
+    buffersize /= mpainfo.decoded_sample_size;
+    while (skip > 0) {
       int32_t xx= skip;
       if (xx > buffersize) xx = buffersize;
       skip -= xx;
@@ -1171,7 +1171,7 @@ static int32_t filinit(CSOUND *csound, LOADER *pp)
 
 void *loader_thread(void *p){
     LOADER *pp = (LOADER *) p;
-    if(filinit(pp->p.csound,pp) == NOTOK) {
+    if(filinit(pp->p.csound,pp) != OK) {
       if(pp->p.error > 0)
         pp->p.csound->Message(pp->p.csound, Str("mp3scal_load error: %s\n"),
                               mp3dec_error(pp->p.error));
@@ -2052,7 +2052,7 @@ static int32_t filinit2(CSOUND *csound, LOADER *pp)
 
 void *loader_thread2(void *p){
     LOADER *pp = (LOADER *) p;
-    if(filinit2(pp->p.csound,pp) == NOTOK) {
+    if(filinit2(pp->p.csound,pp) != OK) {
       if(pp->p.error > 0)
         pp->p.csound->Message(pp->p.csound, Str("mp3scal_load error: %s\n"),
                               mp3dec_error(pp->p.error));
