@@ -48,6 +48,28 @@ typedef struct dats {
 } DATASPACE;
 
 
+typedef struct dats1 {
+  OPDS h;
+  MYFLT *out[1], *time, *kamp, *kpitch, *knum, *klock, *iN,
+    *idecim, *konset, *offset, *dbthresh;
+  int32_t cnt, hsize, curframe, N, decim,tscale;
+  uint32_t nchans;
+  double pos;
+  MYFLT accum;
+  AUXCH outframe[MAXOUTS], win, bwin[MAXOUTS], fwin[MAXOUTS],
+    nwin[MAXOUTS], prev[MAXOUTS], framecount[MAXOUTS], fdata;
+  MYFLT *indata[2];
+  MYFLT *tab;
+  int32_t curbuf;
+  SNDFILE *sf;
+  FDCH    fdch;
+  MYFLT resamp;
+  double tstamp, incr;
+  void *fwdsetup, *invsetup;
+} DATASPACEM;
+
+
+
 static inline int32 intpowint(int32 x, uint32 n) /* Binary +ve power function */
 {
     int32 ans = 1;
@@ -128,11 +150,85 @@ static int32_t sinit(CSOUND *csound, DATASPACE *p)
     return OK;
 }
 
+static int32_t sinitm(CSOUND *csound, DATASPACEM *p)
+{
+    int32_t N =  *p->iN, ui;
+    uint32_t nchans, i;
+    uint32_t size;
+    int32_t decim = *p->idecim;
+
+    if (N) {
+      for (i=0; N; i++) {
+        N >>= 1;
+      }
+      N = intpowint(2, i-1);  /* faster than pow fn */
+    } else N = 2048;
+    if (decim == 0) decim = 4;
+
+    p->hsize = N/decim;
+    p->cnt = p->hsize;
+    p->curframe = 0;
+    p->pos = 0;
+
+    nchans = p->nchans;
+
+    if (UNLIKELY(nchans < 1 || nchans > MAXOUTS))
+      return csound->InitError(csound, Str("invalid number of output arguments"));
+    p->nchans = nchans;
+
+    for (i=0; i < nchans; i++) {
+
+      size = (N+2)*sizeof(MYFLT);
+      if (p->fwin[i].auxp == NULL || p->fwin[i].size < size)
+        csound->AuxAlloc(csound, size, &p->fwin[i]);
+      if (p->bwin[i].auxp == NULL || p->bwin[i].size < size)
+        csound->AuxAlloc(csound, size, &p->bwin[i]);
+      if (p->prev[i].auxp == NULL || p->prev[i].size < size)
+        csound->AuxAlloc(csound, size, &p->prev[i]);
+      size = decim*sizeof(int32_t);
+      if (p->framecount[i].auxp == NULL || p->framecount[i].size < size)
+        csound->AuxAlloc(csound, size, &p->framecount[i]);
+      {
+        int32_t k=0;
+        for (k=0; k < decim; k++) {
+          ((int32_t *)(p->framecount[i].auxp))[k] = k*N;
+        }
+      }
+      size = decim*sizeof(MYFLT)*N;
+      if (p->outframe[i].auxp == NULL || p->outframe[i].size < size)
+        csound->AuxAlloc(csound, size, &p->outframe[i]);
+      else
+        memset(p->outframe[i].auxp,0,size);
+    }
+    size = N*sizeof(MYFLT);
+    if (p->win.auxp == NULL || p->win.size < size)
+      csound->AuxAlloc(csound, size, &p->win);
+
+    {
+      MYFLT x = FL(2.0)*PI_F/N;
+      for (ui=0; ui < N; ui++)
+        ((MYFLT *)p->win.auxp)[ui] = FL(0.5) - FL(0.5)*COS((MYFLT)ui*x);
+    }
+
+    p->N = N;
+    p->decim = decim;
+
+    p->fwdsetup = csound->RealFFT2Setup(csound, N, FFT_FWD);
+    p->invsetup = csound->RealFFT2Setup(csound, N, FFT_INV);
+
+    return OK;
+}
+
 static int32_t sinit1(CSOUND *csound, DATASPACE *p) {
     p->nchans = csound->GetOutputArgCnt(p);
     return sinit(csound, p);
 }
 
+static int32_t sinit1m(CSOUND *csound, DATASPACEM *p) {
+    p->nchans = 1;
+    return sinitm(csound, p);
+}
+
 static int32_t sprocess1(CSOUND *csound, DATASPACE *p)
 {
     uint32_t offset = p->h.insdshead->ksmps_offset;
@@ -144,7 +240,7 @@ static int32_t sprocess1(CSOUND *csound, DATASPACE *p)
     int32_t N = p->N, hsize = p->hsize, cnt = p->cnt, nchans = p->nchans;
     int32_t nsmps = CS_KSMPS, n;
     int32_t sizefrs, size, post, i, j;
-    int64_t spos = p->pos;
+    int64_t spos;  // = p->pos;
     double pos;
     MYFLT *fwin, *bwin, in,
       *prev, *win = (MYFLT *) p->win.auxp;
@@ -175,6 +271,8 @@ static int32_t sprocess1(CSOUND *csound, DATASPACE *p)
         double tim;
         double resamp;
         ft = csound->FTnp2Finde(csound,p->knum);
+        if (UNLIKELY(ft==NULL))
+          return csound->PerfError(csound, &(p->h), Str("function table not found"));
         resamp = ft->gen01args.sample_rate/CS_ESR;
         pitch *= resamp;
         tab = ft->ftable;
@@ -335,6 +433,228 @@ static int32_t sprocess1(CSOUND *csound, DATASPACE *p)
     return OK;
 }
 
+
+static int32_t sprocess1m(CSOUND *csound, DATASPACEM *p)
+{
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    MYFLT pitch = *p->kpitch, *time = p->time, lock = *p->klock,
+      *out, amp =*p->kamp;
+    MYFLT *tab, frac;
+    FUNC *ft;
+    int32_t N = p->N, hsize = p->hsize, cnt = p->cnt, nchans = p->nchans;
+    int32_t nsmps = CS_KSMPS, n;
+    int32_t sizefrs, size, post, i, j;
+    int64_t spos; //= p->pos;
+    double pos;
+    MYFLT *fwin, *bwin, in,
+      *prev, *win = (MYFLT *) p->win.auxp;
+    MYFLT *outframe;
+    MYFLT ph_real, ph_im, tmp_real, tmp_im, div;
+    int32_t *framecnt;
+    int32_t curframe = p->curframe, decim = p->decim;
+    double scaling = (8./decim)/3.;
+
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      for (j=0; j < nchans; j++) {
+        out = p->out[j];
+        memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+      }
+    }
+    if (UNLIKELY(offset)) {
+      for (j=0; j < nchans; j++) {
+        out = p->out[j];
+        memset(out, '\0', offset*sizeof(MYFLT));
+      }
+    }
+
+    for (n=offset; n < nsmps; n++) {
+
+      if (cnt == hsize) {
+        /* audio samples are stored in a function table */
+        double tim;
+        double resamp;
+        ft = csound->FTnp2Finde(csound,p->knum);
+        if (UNLIKELY(ft==NULL))
+          return csound->PerfError(csound, &(p->h), Str("function table not found"));
+        resamp = ft->gen01args.sample_rate/CS_ESR;
+        pitch *= resamp;
+        tab = ft->ftable;
+        size = ft->flen;
+
+        if (UNLIKELY((int32_t) ft->nchanls != nchans))
+          return csound->PerfError(csound, &(p->h), Str("number of output arguments "
+                                       "inconsistent with number of "
+                                       "sound file channels"));
+
+
+
+        /* spos is the reading position in samples, hsize is hopsize,
+           time[n] is current read position in secs
+           esr is sampling rate
+        */
+        tim = time[n]*resamp;
+        spos  = hsize*(int64_t)(tim*CS_ESR/hsize);
+        sizefrs = size/nchans;
+        while (spos > sizefrs) spos -= sizefrs;
+        while (spos <= 0)  spos += sizefrs;
+
+        for (j = 0; j < nchans; j++) {
+          pos = spos;
+          bwin = (MYFLT *) p->bwin[j].auxp;
+          fwin = (MYFLT *) p->fwin[j].auxp;
+          prev = (MYFLT *)p->prev[j].auxp;
+          framecnt  = (int32_t *)p->framecount[j].auxp;
+          outframe= (MYFLT *) p->outframe[j].auxp;
+          /* this loop fills two frames/windows with samples from table,
+             reading is linearly-interpolated,
+             frames are separated by 1 hopsize
+          */
+          for (i=0; i < N; i++) {
+            /* front window, fwin */
+            post = (int32_t) pos;
+            frac = pos  - post;
+            post *= nchans;
+            post += j;
+            while (post < 0) post += size;
+            while (post >= size) post -= size;
+            if (post+nchans <  size)
+              in = tab[post] + frac*(tab[post+nchans] - tab[post]);
+            else in = tab[post];
+
+            fwin[i] = in * win[i]; /* window it */
+            /* back windo, bwin */
+            post = (int32_t) (pos - hsize*pitch);
+            post *= nchans;
+            post += j;
+            while (post < 0) post += size;
+            while (post >= size) post -= size;
+            if (post+nchans <  size)
+              in = tab[post] + frac*(tab[post+nchans] - tab[post]);
+            else in = tab[post];
+            bwin[i] = in * win[i];  /* window it */
+            /* increment read pos according to pitch transposition */
+            pos += pitch;
+          }
+
+          /* take the FFT of both frames
+             re-order Nyquist bin from pos 1 to N
+          */
+          csound->RealFFT2(csound, p->fwdsetup, bwin);
+          bwin[N] = bwin[1];
+          bwin[N+1] = 0.0;
+          csound->RealFFT2(csound,  p->fwdsetup, fwin);
+          fwin[N] = fwin[1];
+          fwin[N+1] = 0.0;
+
+          /* phase vocoder processing */
+
+          for (i=0; i < N + 2; i+=2) {
+            /* phases of previous output frame in exponential format,
+               obtained by dividing by magnitude */
+            div =  FL(1.0)/(HYPOT(prev[i], prev[i+1]) + 1e-20);
+            ph_real  =    prev[i]*div;
+            ph_im =       prev[i+1]*div;
+
+            /* back window magnitudes, phase differences between
+               prev and back windows */
+            tmp_real =   bwin[i] * ph_real + bwin[i+1] * ph_im;
+            tmp_im =   bwin[i] * ph_im - bwin[i+1] * ph_real;
+            bwin[i] = tmp_real;
+            bwin[i+1] = tmp_im;
+          }
+
+          for (i=0; i < N + 2; i+=2) {
+            if (lock) {  /* phase-locking */
+              if (i > 0) {
+                if (i < N) {
+                  tmp_real = bwin[i] + bwin[i-2] + bwin[i+2];
+                  tmp_im = bwin[i+1] + bwin[i-1] + bwin[i+3];
+                }
+                else { /* Nyquist */
+                  tmp_real = bwin[i] + bwin[i-2];
+                  tmp_im = FL(0.0);
+                }
+              }
+              else { /* 0 Hz */
+                tmp_real = bwin[i] + bwin[i+2];
+                tmp_im = FL(0.0);
+              }
+            } else { /* no locking */
+              tmp_real = bwin[i];
+              tmp_im = bwin[i+1];
+            }
+
+            tmp_real += 1e-15;
+            div =  FL(1.0)/(HYPOT(tmp_real, tmp_im));
+
+            /* phases of tmp frame */
+            ph_real = tmp_real*div;
+            ph_im = tmp_im*div;
+
+            /* front window mags, phase sum of
+               tmp and front windows */
+            tmp_real =   fwin[i] * ph_real - fwin[i+1] * ph_im;
+            tmp_im =   fwin[i] * ph_im + fwin[i+1] * ph_real;
+
+            /* phase vocoder output */
+            prev[i] = fwin[i] = tmp_real;
+            prev[i+1] = fwin[i+1] = tmp_im;
+          }
+          /* re-order bins and take inverse FFT */
+          fwin[1] = fwin[N];
+          csound->RealFFT2(csound, p->invsetup, fwin);
+          /* frame counter */
+          framecnt[curframe] = curframe*N;
+          /* write to overlapped output frames */
+          for (i=0;i<N;i++) outframe[framecnt[curframe]+i] = win[i]*fwin[i];
+
+        }
+
+        cnt=0;
+        curframe++;
+        if (curframe == decim) curframe = 0;
+      }
+
+      for (j=0; j < nchans; j++) {
+        framecnt  = (int32_t *) p->framecount[j].auxp;
+        outframe  = (MYFLT *) p->outframe[j].auxp;
+        out = p->out[j];
+        out[n] = (MYFLT)0;
+        /* write output */
+        for (i = 0; i < decim; i++) {
+          out[n] += outframe[framecnt[i]];
+          framecnt[i]++;
+        }
+        /* scale output */
+        out[n] *= amp*scaling;
+      }
+      cnt++;
+    }
+
+    p->cnt = cnt;
+    p->curframe = curframe;
+    return OK;
+}
+
+
+
+static int32_t sinit2m(CSOUND *csound, DATASPACEM *p)
+{
+    uint32_t size,i;
+    p->nchans = csound->GetOutputArgCnt(p);
+    sinitm(csound, p);
+    size = p->N*sizeof(MYFLT);
+    for (i=0; i < p->nchans; i++)
+      if (p->nwin[i].auxp == NULL || p->nwin[i].size < size)
+        csound->AuxAlloc(csound, size, &p->nwin[i]);
+    p->pos = *p->offset*CS_ESR + p->hsize;
+    p->tscale  = 0;
+    p->accum = 0;
+    return OK;
+}
+
 static int32_t sinit2(CSOUND *csound, DATASPACE *p)
 {
     uint32_t size,i;
@@ -390,6 +710,8 @@ static int32_t sprocess2(CSOUND *csound, DATASPACE *p)
       if (cnt == hsize) {
         double resamp;
         ft = csound->FTnp2Finde(csound,p->knum);
+        if (UNLIKELY(ft==NULL))
+          return csound->PerfError(csound, &(p->h), Str("function table not found"));
         resamp = ft->gen01args.sample_rate/CS_ESR;
         pitch *= resamp;
         time  *= resamp;
@@ -475,7 +797,228 @@ static int32_t sprocess2(CSOUND *csound, DATASPACE *p)
 
           tmp_real = tmp_im = (MYFLT) 1e-20;
           for (i=2; i < N; i++) {
-            tmp_real += nwin[i]*nwin[i] + nwin[i+1]*nwin[i+1];
+            tmp_real += nwin[i]*nwin[i];
+            if (i+1 < N) tmp_real += nwin[i+1]*nwin[i+1];
+            tmp_im += fwin[i]*fwin[i] + fwin[i+1]*fwin[i+1];
+          }
+          powrat = FL(20.0)*LOG10(tmp_real/tmp_im);
+          if (powrat > dbtresh) p->tscale=0;
+          /*else tscale=1;*/
+
+          fwin[N] = fwin[1];
+          fwin[N+1] = FL(0.0);
+
+          for (i=0; i < N + 2; i+=2) {
+
+            div =  FL(1.0)/(HYPOT(prev[i], prev[i+1]) + 1.0e-20);
+            ph_real  =    prev[i]*div;
+            ph_im =       prev[i+1]*div;
+
+            tmp_real =   bwin[i] * ph_real + bwin[i+1] * ph_im;
+            tmp_im =   bwin[i] * ph_im - bwin[i+1] * ph_real;
+            bwin[i] = tmp_real;
+            bwin[i+1] = tmp_im;
+          }
+
+          for (i=0; i < N + 2; i+=2) {
+            if (lock) {
+              if (i > 0) {
+                if (i < N) {
+                  tmp_real = bwin[i] + bwin[i-2] + bwin[i+2];
+                  tmp_im = bwin[i+1] + bwin[i-1] + bwin[i+3];
+                }
+                else {
+                  tmp_real = bwin[i] + bwin[i-2];
+                  tmp_im = FL(0.0);
+                }
+              }
+              else {
+                tmp_real = bwin[i] + bwin[i+2];
+                tmp_im = FL(0.0);
+              }
+            }
+            else {
+              tmp_real = bwin[i];
+              tmp_im = bwin[i+1];
+            }
+
+            tmp_real += 1e-15;
+            div =  FL(1.0)/(HYPOT(tmp_real, tmp_im));
+
+            ph_real = tmp_real*div;
+            ph_im = tmp_im*div;
+
+            tmp_real =   fwin[i] * ph_real - fwin[i+1] * ph_im;
+            tmp_im =   fwin[i] * ph_im + fwin[i+1] * ph_real;
+
+            prev[i] = fwin[i] = tmp_real;
+            prev[i+1] = fwin[i+1] = tmp_im;
+          }
+
+          fwin[1] = fwin[N];
+          csound->RealFFT2(csound, p->invsetup, fwin);
+
+          framecnt[curframe] = curframe*N;
+
+          for (i=0;i<N;i++) outframe[framecnt[curframe]+i] = win[i]*fwin[i];
+
+        }
+        cnt=0;
+        curframe++;
+        if (curframe == decim) curframe = 0;
+      }
+
+      for (j=0; j < nchans; j++) {
+        out = p->out[j];
+        framecnt  = (int32_t *) p->framecount[j].auxp;
+        outframe  = (MYFLT *) p->outframe[j].auxp;
+
+        out[n] = (MYFLT) 0;
+
+        for (i = 0; i < decim; i++) {
+          out[n] += outframe[framecnt[i]];
+          framecnt[i]++;
+        }
+        out[n] *= amp*scaling;
+      }
+      cnt++;
+    }
+    p->cnt = cnt;
+    p->curframe = curframe;
+    p->pos = spos;
+    return OK;
+
+}
+
+static int32_t sprocess2m(CSOUND *csound, DATASPACEM *p)
+{
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    MYFLT pitch = *p->kpitch, time = *p->time, lock = *p->klock;
+    MYFLT *out, amp =*p->kamp;
+    MYFLT *tab,frac,  dbtresh = *p->dbthresh;
+    FUNC *ft;
+    int32_t N = p->N, hsize = p->hsize, cnt = p->cnt, sizefrs, nchans = p->nchans;
+    int32_t  nsmps = CS_KSMPS, n;
+    int32_t size, post, i, j;
+    double pos, spos = p->pos;
+    MYFLT *fwin, *bwin;
+    MYFLT in, *nwin, *prev;
+    MYFLT *win = (MYFLT *) p->win.auxp, *outframe;
+    MYFLT powrat;
+    MYFLT ph_real, ph_im, tmp_real, tmp_im, div;
+    int32_t *framecnt, curframe = p->curframe;
+    int32_t decim = p->decim;
+    double scaling = (8./decim)/3.;
+
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      for (j=0; j < nchans; j++) {
+        out = p->out[j];
+        memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+      }
+    }
+    if (UNLIKELY(offset)) {
+      for (j=0; j < nchans; j++) {
+        out = p->out[j];
+        memset(out, '\0', offset*sizeof(MYFLT));
+      }
+    }
+
+    for (n=offset; n < nsmps; n++) {
+
+      if (cnt == hsize) {
+        double resamp;
+        ft = csound->FTnp2Finde(csound,p->knum);
+        if (UNLIKELY(ft==NULL))
+          return csound->PerfError(csound, &(p->h), Str("function table not found"));
+        resamp = ft->gen01args.sample_rate/CS_ESR;
+        pitch *= resamp;
+        time  *= resamp;
+        tab = ft->ftable;
+        size = ft->flen;
+
+        if (time < 0 || time >= 1 || !*p->konset) {
+          spos += hsize*time;
+          //csound->Message(csound, "position: %f \n", spos);
+        }
+        else if (p->tscale) {
+          spos += hsize*(time/(1+p->accum));
+          p->accum=0.0;
+        }
+        else  {
+          spos += hsize;
+          p->accum++;
+          p->tscale = 1;
+        }
+        if (UNLIKELY((int32_t) ft->nchanls != nchans))
+          return csound->PerfError(csound, &(p->h),
+                                   Str("number of output arguments "
+                                       "inconsistent with number of "
+                                       "sound file channels"));
+
+        sizefrs = size/nchans;
+        while (spos > sizefrs) spos -= sizefrs;
+        while (spos <= 0)  spos += sizefrs;
+
+
+        for (j = 0; j < nchans; j++) {
+          pos = spos;
+          bwin = (MYFLT *) p->bwin[j].auxp;
+          fwin = (MYFLT *) p->fwin[j].auxp;
+          nwin = (MYFLT *) p->nwin[j].auxp;
+          prev = (MYFLT *)p->prev[j].auxp;
+          framecnt  = (int32_t *)p->framecount[j].auxp;
+          outframe= (MYFLT *) p->outframe[j].auxp;
+
+          for (i=0; i < N; i++) {
+            post = (int32_t) pos;
+            frac = pos  - post;
+            post *= nchans;
+            post += j;
+
+            while (post < 0) post += size;
+            while (post >= size) post -= size;
+            if (post+nchans <  size)
+              in = tab[post] + frac*(tab[post+nchans] - tab[post]);
+            else in = tab[post];
+
+            fwin[i] = in * win[i];
+
+            post = (int32_t) (pos - hsize*pitch);
+            post *= nchans;
+            post += j;
+            while (post < 0) post += size;
+            while (post >= size) post -= size;
+            //if (post >= 0 && post < size)
+            if (post+nchans <  size)
+              in =  tab[post] + frac*(tab[post+nchans] - tab[post]);
+            else in = tab[post];
+            //else in =  (MYFLT) 0;
+
+            bwin[i] = in * win[i];
+            post = (int32_t) pos + hsize;
+            post *= nchans;
+            post += j;
+            while (post < 0) post += size;
+            while (post >= size) post -= size;
+            if (post+nchans <  size)
+              in = tab[post] + frac*(tab[post+nchans] - tab[post]);
+            else in = tab[post];
+            nwin[i] = in * win[i];
+            pos += pitch;
+          }
+
+          csound->RealFFT2(csound, p->fwdsetup, bwin);
+          bwin[N] = bwin[1];
+          bwin[N+1] = FL(0.0);
+          csound->RealFFT2(csound, p->fwdsetup,  fwin);
+          csound->RealFFT2(csound,  p->fwdsetup, nwin);
+
+          tmp_real = tmp_im = (MYFLT) 1e-20;
+          for (i=2; i < N; i++) {
+            tmp_real += nwin[i]*nwin[i];
+            if (i+1 < N) tmp_real += nwin[i+1]*nwin[i+1];
             tmp_im += fwin[i]*fwin[i] + fwin[i+1]*fwin[i+1];
           }
           powrat = FL(20.0)*LOG10(tmp_real/tmp_im);
@@ -1110,8 +1653,8 @@ int32_t am_fm(CSOUND *csound, AMFM *p) {
     }
 
     for (n=offset; n < nsmps; n++) {
-      am[n] = SQRT(re[n]*re[n] + im[n]*im[n]);
-      ph = atan2(im[n], re[n]);
+      am[n] = HYPOT(re[n], im[n]);
+      ph = ATAN2(im[n], re[n]);
       f = ph - oph;
       oph = ph;
       if (f >= PI) f -= 2*PI;
@@ -1125,8 +1668,12 @@ int32_t am_fm(CSOUND *csound, AMFM *p) {
 
 static OENTRY pvlock_localops[] =
   {
+   {"mincer", sizeof(DATASPACEM), 0, 3, "a", "akkkkoo",
+    (SUBR)sinit1m,(SUBR)sprocess1m },
    {"mincer", sizeof(DATASPACE), 0, 3, "mm", "akkkkoo",
     (SUBR)sinit1,(SUBR)sprocess1 },
+   {"temposcal", sizeof(DATASPACEM), 0, 3, "a", "kkkkkooPOP",
+    (SUBR)sinit2m,(SUBR)sprocess2m },
    {"temposcal", sizeof(DATASPACE), 0, 3, "mm", "kkkkkooPOP",
     (SUBR)sinit2,(SUBR)sprocess2 },
    {"filescal", sizeof(DATASPACE), 0, 3, "mm", "kkkSkooPOP",
