@@ -54,16 +54,12 @@ static int32_t syncgrain_init(CSOUND *csound, syncgrain *p)
       p->olaps = 2;
 
     size =  (p->olaps) * sizeof(double);
-    if (p->index.auxp == NULL || p->index.size < (uint32_t)size)
-      csound->AuxAlloc(csound, size, &p->index);
-    if (p->envindex.auxp == NULL || p->envindex.size < (uint32_t)size)
-      csound->AuxAlloc(csound, size, &p->envindex);
-    if (p->envincr.auxp == NULL || p->envincr.size < (uint32_t)size)
-      csound->AuxAlloc(csound, size, &p->envincr);
-
+    csound->AuxAlloc(csound, size, &p->index);
+    csound->AuxAlloc(csound, size, &p->envindex);
+    csound->AuxAlloc(csound, size, &p->envincr);
     size = (p->olaps) * sizeof(int32_t);
-    if (p->streamon.auxp == NULL || p->streamon.size < (uint32_t)size)
-      csound->AuxAlloc(csound, size, &p->streamon);
+    csound->AuxAlloc(csound, size, &p->streamon);
+ 
 
     p->count = 0;                  /* sampling period counter */
 
@@ -84,11 +80,12 @@ static int32_t syncgrain_process(CSOUND *csound, syncgrain *p)
     MYFLT   *output = p->output;
     MYFLT   *datap = p->sfunc->ftable;
     MYFLT   *ftable = p->efunc->ftable;
-    int32_t     *streamon = (int32_t *) p->streamon.auxp;
+
     float   start = p->start, frac = p->frac;
     double  *index = (double *) p->index.auxp;
     double  *envindex = (double *) p->envindex.auxp;
     double  *envincrn = (double *) p->envincr.auxp;
+    int32_t     *streamon = (int32_t *) p->streamon.auxp;
     uint32_t offset = p->h.insdshead->ksmps_offset;
     uint32_t early  = p->h.insdshead->ksmps_no_end;
     uint32_t vecpos, vecsize=CS_KSMPS;
@@ -438,6 +435,7 @@ static int32_t filegrain_init(CSOUND *csound, filegrain *p)
                        (p->dataframes+1)*sizeof(MYFLT)*p->nChannels, &p->buffer);
 
     buffer = (MYFLT *) p->buffer.auxp;
+    memset(&sfinfo, '\0', sizeof(sfinfo)); /* for Valgrind */
     /* open file and read the first block using *p->ioff */
     fd = csound->FileOpen2(csound, &(p->sf), CSFILE_SND_R, fname, &sfinfo,
                             "SFDIR;SSDIR", CSFTYPE_UNKNOWN_AUDIO, 0);
