@@ -83,9 +83,11 @@ void do_logging(char *s)
     if (!strcmp(s, "NULL") || !strcmp(s, "null"))
       nomessages = 1;
     else if ((logFile = fopen(s, "w")) == NULL) {
+      #ifndef __wasi__
       fprintf(stderr, Str("Error opening log file '%s': %s\n"),
               s, strerror(errno));
       exit(1);
+      #endif
     }
     /* if logging to file, set message callback */
     if (logFile != NULL)
@@ -245,7 +247,7 @@ static const char *longUsageList[] = {
   " ",
   Str_noop("--m-amps=[01]           messages on note amps"),
   Str_noop("--m-range=[01]          messages on range errors"),
-  Str_noop("--m-warnings=[01]       mesage on warnings"),
+  Str_noop("--m-warnings=[01]       message on warnings"),
   Str_noop("--m-raw=[01]            raw amp messages"),
   Str_noop("--m-dB=[01]             amp messages in dB"),
   Str_noop("--m-colours=[01]        colour amp messages"),
@@ -255,7 +257,9 @@ static const char *longUsageList[] = {
                                    "ORC/SCO-relative line #s"),
   Str_noop("--extract-score=FNAME   extract from score.srt using extract file"),
   Str_noop("--keep-sorted-score"),
-  Str_noop("--simple_sorted_score"),
+  Str_noop("--keep-sorted-score=FNAME"),
+  Str_noop("--simple-sorted-score"),
+  Str_noop("--simple-sorted-score=FNAME"),
   Str_noop("--env:NAME=VALUE        set environment variable NAME to VALUE"),
   Str_noop("--env:NAME+=VALUE       append VALUE to environment variable NAME"),
   Str_noop("--strsetN=VALUE         set strset table at index N to VALUE"),
@@ -269,6 +273,7 @@ static const char *longUsageList[] = {
   Str_noop("--sched                 set real-time scheduling priority and "
                                    "lock memory"),
   Str_noop("--sched=N               set priority to N and lock memory"),
+  Str_noop("--opcode-dir=DIR        load all plugins from DIR"),
   Str_noop("--opcode-lib=NAMES      dynamic libraries to load"),
   Str_noop("--opcode-omit=NAMES     dynamic libraries not to load"),
   Str_noop("--omacro:XXX=YYY        set orchestra macro XXX to value YYY"),
@@ -310,6 +315,7 @@ static const char *longUsageList[] = {
                                    "PFFFT = 1, vDSP =2)"),
   Str_noop("--udp-echo              echo UDP commands on terminal"),
   Str_noop("--aft-zero              set aftertouch to zero, not 127 (default)"),
+  Str_noop("--limiter[=num]         include clipping in audio output"),
   " ",
   Str_noop("--help                  long help"),
   NULL
@@ -321,6 +327,8 @@ void print_short_usage(CSOUND *csound)
     char    buf[256];
     int     i;
     i = -1;
+    print_csound_version(csound);
+    csound->Message(csound, Str("\nShort options format:\n"));
     while (shortUsageList[++i] != NULL) {
       snprintf(buf, 256, "%s\n", shortUsageList[i]);
       csound->Message(csound, "%s", Str(buf));
@@ -333,14 +341,13 @@ void print_short_usage(CSOUND *csound)
 static void longusage(CSOUND *p)
 {
     const char **sp;
+    print_short_usage(p);
     p->Message(p, Str("Usage:     csound [-flags] orchfile scorefile\n"));
     p->Message(p, Str("Legal flags are:\n"));
     p->Message(p, Str("Long format:\n\n"));
     for (sp = &(longUsageList[0]); *sp != NULL; sp++)
       p->Message(p, "%s\n", Str(*sp));
     dump_cfg_variables(p);
-    p->Message(p, Str("\nShort format:\n\n"));
-    print_short_usage(p);
 }
 
 CS_NORETURN void dieu(CSOUND *csound, char *s, ...)
@@ -353,7 +360,7 @@ CS_NORETURN void dieu(CSOUND *csound, char *s, ...)
     va_start(args, s);
     csound->ErrMsgV(csound, Str("Csound Command ERROR:    "), s, args);
     va_end(args);
-    //***FIXME This code makes no sense
+    //// FIXME This code makes no sense
     /* if (csound->info_message_request == 0) { */
     /*   csound->info_message_request = 0; */
     /*   csound->LongJmp(csound, 1); */
@@ -510,7 +517,7 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       nn->mac = s;
       nn->next = csound->omacros;
       csound->omacros = nn;
-      return 1;
+     return 1;
     }
     else if (!(strncmp(s, "smacro:", 7))) {
       if (csound->orcname_mode) return 1;
@@ -569,7 +576,7 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       if (UNLIKELY(*s=='\0')) dieu(csound, Str("no hardware bufsamps"));
       O->inbufsamps = O->outbufsamps = atoi(s);
       return 1;
-    }
+   }
     else if (!(strcmp (s, "orc"))) {
       csound->use_only_orchfile = 1;    /* orchfile without scorefile */
       return 1;
@@ -738,8 +745,8 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       s += 7;
       if (UNLIKELY(*s=='\0')) dieu(csound, Str("no message amps"));
       sscanf(s, "%d", &n);
-      if (n) O->msglevel |= AMPLMSG;
-      else O->msglevel &= ~AMPLMSG;
+      if (n) O->msglevel |= CS_AMPLMSG;
+      else O->msglevel &= ~CS_AMPLMSG;
       return 1;
     }
     else if (!(strncmp (s, "m-range=",8))) {
@@ -747,8 +754,8 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       s += 8;
       if (UNLIKELY(*s=='\0')) dieu(csound, Str("no message range"));
       sscanf(s, "%d", &n);
-      if (n) O->msglevel |= RNGEMSG;
-      else O->msglevel &= ~RNGEMSG;
+      if (n) O->msglevel |= CS_RNGEMSG;
+      else O->msglevel &= ~CS_RNGEMSG;
       return 1;
      }
     else if (!(strncmp (s, "m-warnings=",11))) {
@@ -756,8 +763,8 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       s += 11;
       if (UNLIKELY(*s=='\0')) dieu(csound, Str("no message warnings"));
       sscanf(s, "%d", &n);
-      if (n) O->msglevel |= WARNMSG;
-      else O->msglevel &= ~WARNMSG;
+      if (n) O->msglevel |= CS_WARNMSG;
+      else O->msglevel &= ~CS_WARNMSG;
       return 1;
     }
     else if (!(strncmp (s, "m-raw=",6))) {
@@ -774,8 +781,8 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       s += 5;
       if (UNLIKELY(*s=='\0')) dieu(csound, Str("no message dB"));
       sscanf(s, "%d", &n);
-      if (n) O->msglevel |= RAWMSG;
-      else O->msglevel &= ~RAWMSG;
+      if (n) O->msglevel |= CS_RAWMSG;
+      else O->msglevel &= ~CS_RAWMSG;
       return 1;
     }
     else if (!(strncmp (s, "m-colours=",10)) || !(strncmp (s, "m-colors=",9))) {
@@ -792,8 +799,8 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       s += 13;
       if (UNLIKELY(*s=='\0')) dieu(csound, Str("no benchmark level"));
       sscanf(s, "%d", &n);
-      if (n) O->msglevel |= TIMEMSG;
-      else O->msglevel &= ~TIMEMSG;
+      if (n) O->msglevel |= CS_TIMEMSG;
+      else O->msglevel &= ~CS_TIMEMSG;
       return 1;
     }
     else if (!(strncmp (s, "csd-line-nums=",14))) {
@@ -849,6 +856,10 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       O->sfwrite = 1;
       return 1;
     }
+    else if (!(strcmp (s, "print_version"))) {
+      csound->print_version = 1;
+      return 1;
+    }
     else if (!(strncmp (s, "logfile=", 8))) {
       s += 8;
       if (UNLIKELY(*s=='\0')) dieu(csound, Str("no log file"));
@@ -877,8 +888,20 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       return 1;
     }
     /* -t0 */
+    else if (!(strncmp(s, "keep-sorted-score=", 18))) {
+      s += 18;
+      csound->score_srt = s;
+      csound->keep_tmp= 1;
+      return 1;
+    }
     else if (!(strcmp (s, "keep-sorted-score"))) {
-      csound->keep_tmp = 1;
+      csound->keep_tmp= 1;
+      return 1;
+    }
+    else if (!(strncmp (s, "simple-sorted-score=", 20))) {
+      s +=20;
+      csound->score_srt = s;
+      csound->keep_tmp = 2;
       return 1;
     }
     else if (!(strcmp (s, "simple-sorted-score"))) {
@@ -997,6 +1020,11 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       O->midiVelocityAmp = atoi(s);
       return 1;
     }
+    else if (!(strncmp (s, "opcode-dir=", 11))){
+        s += 11;
+        csoundLoadPlugins(csound, s);
+        return 1;
+    }
     else if (!(strncmp (s, "opcode-lib=", 11))) {
       int   nbytes;
       s += 11;
@@ -1036,7 +1064,7 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       return 1;
     }
     else if (!(strcmp(s, "version"))) {
-      //print_csound_version(csound); // as already printed!
+      print_csound_version(csound);
       csound->LongJmp(csound, 0);
     }
     else if (!(strcmp(s, "help"))) {
@@ -1097,7 +1125,7 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       return 1;
     }
     else if (!(strncmp(s, "udp-echo",8))) {
-      s += 8;
+      //s += 8;
       O->echo = 1;
       return 1;
     }
@@ -1175,29 +1203,51 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
         /* these are default values to get the
            backend to open successfully */
         set_output_format(O, 'f');
+        O->sr_override = FL(-1.0);
         O->inbufsamps = O->outbufsamps = 256;
         O->oMaxLag = 1024;
         csoundLoadExternals(csound);
         if (csoundInitModules(csound) != 0)
           csound->LongJmp(csound, 1);
         sfopenout(csound);
-        csound->MessageS(csound,CSOUNDMSG_STDOUT,
+        csound->MessageS(csound, CSOUNDMSG_STDOUT,
                          "system sr: %f\n", csound->system_sr(csound,0));
         sfcloseout(csound);
+        // csound->LongJmp(csound, 0);
       }
       csound->info_message_request = 1;
       return 1;
     }
-    else if(!strncmp(s, "use-system-sr",13)){
+    else if(!strncmp(s, "use-system-sr",13)) {
       if (O->sr_override == FL(0.0))
           O->sr_override = FL(-1.0);
       return 1;
     }
-    else if (!(strcmp(s, "aft-zero"))){
-      csound->aftouch = 0;
+    else if(!strncmp(s, "default-ksmps=",14)) {
+      s += 14;
+      O->kr_default = O->sr_default/atof(s);
       return 1;
     }
 
+    else if (!(strcmp(s, "aft-zero"))) {
+      csound->aftouch = 0;
+      return 1;
+    }
+    else if (!(strncmp(s, "limiter=", 8)))  {
+      s += 8;
+      O->limiter = atof(s);
+      if (O->limiter>1.0 || O->limiter<0) {
+        csound->MessageS(csound, CSOUNDMSG_STDOUT,
+                         Str("Ignoring invalid limiter\n"));
+        O->limiter = 0;
+      }
+      return 1;
+
+    }
+    else if (!(strcmp(s, "limiter"))) {
+      O->limiter = 0.5;
+      return 1;
+    }
     csoundErrorMsg(csound, Str("unknown long option: '--%s'"), s);
     return 0;
 }
@@ -1225,7 +1275,7 @@ PUBLIC int argdecode(CSOUND *csound, int argc, const char **argv_)
       strcpy(p2, argv_[i]);
       p2 = (char*) p2 + ((int) strlen(argv_[i]) + 1);
     }
-    csound->keep_tmp = 0;
+    //<csound->keep_tmp = 0;
 
     do {
       s = *++argv;
@@ -1812,15 +1862,19 @@ static void list_audio_devices(CSOUND *csound, int output){
     CS_AUDIODEVICE *devs = (CS_AUDIODEVICE *)
       csound->Malloc(csound, n*sizeof(CS_AUDIODEVICE));
     if (output)
-      csound->MessageS(csound,CSOUNDMSG_STDOUT,
-                       Str("%d audio output devices\n"), n);
+      csound->Message(csound, Str("%d audio output devices\n"), n);
     else
-      csound->MessageS(csound, CSOUNDMSG_STDOUT,
-                       Str("%d audio input devices\n"), n);
+      csound->Message(csound, Str("%d audio input devices\n"), n);
     csoundGetAudioDevList(csound,devs,output);
-    for (i=0; i < n; i++)
-      csound->Message(csound, " %d: %s (%s)\n",
-                      i, devs[i].device_id, devs[i].device_name);
+    for (i=0; i < n; i++) {
+      int nchnls = devs[i].max_nchnls;
+      if(nchnls > 0)
+        csound->Message(csound, " %d: %s (%s) [ch:%d]\n",
+                        i, devs[i].device_id, devs[i].device_name, nchnls);
+      else
+        csound->Message(csound, " %d: %s (%s)\n",
+                        i, devs[i].device_id, devs[i].device_name);
+    }
     csound->Free(csound, devs);
 }
 
