@@ -72,8 +72,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <errno.h>
+
+#if !(defined (__wasi__))
 #include <setjmp.h>
+#include <errno.h>
+#endif
 
 #include "csoundCore.h"
 #include "csmodule.h"
@@ -88,7 +91,7 @@
 #endif
 #endif
 
-#if !(defined (NACL))
+#if !(defined (NACL)) && !(defined (__wasi__))
 #if defined(LINUX) || defined(NEW_MACH_CODE) || defined(__HAIKU__)
 #include <dlfcn.h>
 #elif defined(WIN32)
@@ -135,7 +138,7 @@ static  const   char    *plugindir_envvar =   "OPCODE6DIR";
 static  const   char    *plugindir64_envvar = "OPCODE6DIR64";
 
 /* default directory to load plugins from if environment variable is not set */
-#if !(defined (NACL))
+#if !(defined (NACL)) && !(defined (__wasi__))
 #ifdef __HAIKU__
 # ifndef USE_DOUBLE
    static char haikudirs[] = "/boot/system/lib/csound6/plugins:"
@@ -172,7 +175,7 @@ static  const   char    *plugindir64_envvar = "OPCODE6DIR64";
 #endif
 
 typedef struct opcodeLibFunc_s {
-    long    (*opcode_init)(CSOUND *, OENTRY **);  /* list of opcode entries  */
+    int64_t  (*opcode_init)(CSOUND *, OENTRY **);  /* list of opcode entries  */
     NGFENS  *(*fgen_init)(CSOUND *);        /* list of named GEN routines    */
     void    (*dummy)(void);                 /* unused                        */
 } opcodeLibFunc_t;
@@ -232,6 +235,147 @@ static int check_plugin_compatibility(CSOUND *csound, const char *fname, int n)
     return 0;
 }
 
+
+/**
+ * Initialise a single module.
+ * Return value is CSOUND_SUCCESS if there was no error.
+ */
+static CS_NOINLINE int csoundInitModule(CSOUND *csound, csoundModule_t *m)
+{
+    int     i;
+
+    if (m->PreInitFunc != NULL) {
+      if (m->fn.p.InitFunc != NULL) {
+        i = m->fn.p.InitFunc(csound);
+        if (UNLIKELY(i != 0)) {
+          print_module_error(csound, Str("Error starting module '%s'"),
+                                     &(m->name[0]), m, i);
+          return CSOUND_ERROR;
+        }
+      }
+    }
+    else {
+      /* deal with fgens if there are any */
+      if (m->fn.o.fgen_init != NULL) {
+        NGFENS  *names = m->fn.o.fgen_init(csound);
+        for (i = 0; names[i].name != NULL; i++)
+          allocgen(csound, names[i].name, names[i].fn);
+      }
+      if (m->fn.o.opcode_init != NULL) {
+        OENTRY  *opcodlst_n;
+        int64_t    length;
+        /* load opcodes */
+        if (UNLIKELY((length = m->fn.o.opcode_init(csound, &opcodlst_n)) < 0L))
+          return CSOUND_ERROR;
+        else {
+          length /= (long) sizeof(OENTRY);
+          if (length) {
+            if (UNLIKELY(csoundAppendOpcodes(csound, opcodlst_n,
+                                               (int) length) != 0))
+              return CSOUND_ERROR;
+          }
+        }
+      }
+    }
+    return CSOUND_SUCCESS;
+}
+
+
+#ifdef __wasi__
+
+__attribute__((used))
+void csoundWasiLoadPlugin(CSOUND *csound, void *preInitFunc, void *initFunc, void *destFunc, void *errCodeToStr) {
+    csoundModule_t *module = csound->Malloc(csound, sizeof(csoundModule_t) + 1);
+    module->h = (void*) NULL;
+
+    // The javascript host must assert that this is provided
+    module->PreInitFunc = (int (*)(CSOUND *)) preInitFunc;
+    if (initFunc) {
+        module->fn.p.InitFunc = (int (*)(CSOUND *)) initFunc;
+    }
+    if (destFunc) {
+        module->fn.p.DestFunc = (int (*)(CSOUND *)) destFunc;
+    }
+    if (errCodeToStr) {
+        module->fn.p.ErrCodeToStr = (const char *(*)(int)) errCodeToStr;
+    }
+
+    module->nxt = (csoundModule_t*) csound->csmodule_db;
+    csound->csmodule_db = module;
+
+    module->PreInitFunc(csound);
+}
+
+__attribute__((used))
+void csoundWasiLoadOpcodeLibrary(CSOUND *csound, void *fgenInitFunc, void *opcodeInitFunc) {
+    csoundModule_t *module = csound->Malloc(csound, sizeof(csoundModule_t) + 1);
+    module->h = (void*) NULL;
+
+    if (fgenInitFunc) {
+        module->fn.o.fgen_init = (NGFENS *(*)(CSOUND *)) fgenInitFunc;
+    }
+
+    if (opcodeInitFunc) {
+        module->fn.o.opcode_init = (int64_t (*)(CSOUND *, OENTRY **)) opcodeInitFunc;
+    }
+
+    module->nxt = (csoundModule_t*) csound->csmodule_db;
+    csound->csmodule_db = module;
+}
+
+__attribute__((used))
+int csoundDestroyModules(CSOUND *csound) {
+    csoundModule_t  *m;
+    int i;
+    int retval = CSOUND_SUCCESS;
+
+    while (csound->csmodule_db != NULL) {
+      m = (csoundModule_t*) csound->csmodule_db;
+      /* call destructor functions */
+      if (m->PreInitFunc != NULL && m->fn.p.DestFunc != NULL) {
+        i = m->fn.p.DestFunc(csound);
+        if (UNLIKELY(i != 0)) {
+          print_module_error(csound, Str("Error de-initialising module '%s'"),
+                                     &(m->name[0]), m, i);
+          retval = CSOUND_ERROR;
+        }
+      }
+      csound->csmodule_db = (void*) m->nxt;
+      /* free memory used by database */
+      csound->Free(csound, (void*) m);
+    }
+    /* return with error code */
+    return retval;
+}
+
+__attribute__((used))
+int csoundInitModules(CSOUND *csound) {
+    csoundModule_t  *m;
+    int i, retval = CSOUND_SUCCESS;
+    for (m = (csoundModule_t*) csound->csmodule_db; m != NULL; m = m->nxt) {
+      i = csoundInitModule(csound, m);
+      if (UNLIKELY(i != CSOUND_SUCCESS && i < retval))
+        retval = i;
+    }
+    /* return with error code */
+    return retval;
+}
+
+// In browser-wasi, this function is replaced
+// by the js-host.
+// extern int csoundLoadModules(CSOUND *csound);
+
+int csoundLoadExternals(CSOUND *csound) {
+    return 0;
+}
+
+int csoundLoadAndInitModules(CSOUND *csound, const char *opdir) {
+    return 0;
+}
+
+#else /* __wasi__ */
+
+
 /* load a single plugin library, and run csoundModuleCreate() if present */
 /* returns zero on success */
 
@@ -321,7 +465,7 @@ static CS_NOINLINE int csoundLoadExternal(CSOUND *csound,
     else {
       /* opcode library */
       m.fn.o.opcode_init =
-          (long (*)(CSOUND *, OENTRY **))
+          (int64_t (*)(CSOUND *, OENTRY **))
               csoundGetLibrarySymbol(h, opcode_init_Name);
       m.fn.o.fgen_init =
           (NGFENS *(*)(CSOUND *)) csoundGetLibrarySymbol(h, fgen_init_Name);
@@ -401,6 +545,15 @@ static int csoundCheckOpcodeDeny(CSOUND * csound, const char *fname)
     return 0;
 }
 
+
+static int _dir_exists(char *path) {
+    // returns 1 if path is a directory and it exists
+    struct stat s;
+    int err = stat(path, &s);
+    return (err == 0 && S_ISDIR(s.st_mode)) ? 1 : 0;
+}
+
+
 /**
  * Load plugin libraries for Csound instance 'csound', and call
  * pre-initialisation functions.
@@ -411,13 +564,15 @@ static int csoundCheckOpcodeDeny(CSOUND * csound, const char *fname)
 int csoundLoadModules(CSOUND *csound)
 {
 #if (defined(HAVE_DIRENT_H) && (TARGET_OS_IPHONE == 0))
-    DIR             *dir;
-    struct dirent   *f;
-    const char      *dname, *fname;
-    char            buf[1024];
-    int             i, n, len, err = CSOUND_SUCCESS;
-    char   *dname1, *end;
-    int     read_directory = 1;
+    DIR *dir;
+    struct dirent *f;
+    const char *dname, *fname;
+    enum { searchpath_buflen = 2048, buflen = 1024 };
+    char buf[buflen];
+    int i, n, len, err = CSOUND_SUCCESS;
+    char *dname1, *end;
+    int read_directory = 1;
+    char searchpath_buf[searchpath_buflen];
     char sep =
 #ifdef WIN32
     ';';
@@ -442,6 +597,297 @@ int csoundLoadModules(CSOUND *csound)
       if (dname == NULL)
 #  endif
 #endif
+#ifdef  CS_DEFAULT_PLUGINDIR
+        dname = CS_DEFAULT_PLUGINDIR;
+  #ifdef __HAIKU__
+        dfltdir = 1;
+  #endif
+#else
+      dname = "";
+#endif
+    }
+
+    /* opcodedir GLOBAL override **experimental** */
+    if (csound->opcodedir != NULL) {
+      dname = csound->opcodedir;
+      csound->Message(csound, "OPCODEDIR overridden to %s \n", dname);
+    }
+    size_t pos = strlen(dname);
+    char *userplugindir = getenv("CS_USER_PLUGINDIR");
+    // The user set a search path for plugins via an env variable. Paths here
+    // should be absolute and should not need variable expansion
+    if(userplugindir != NULL) {
+      snprintf(searchpath_buf, searchpath_buflen, "%s%c%s", dname, sep, userplugindir);
+      dname = searchpath_buf;
+    } else {
+#ifdef CS_DEFAULT_USER_PLUGINDIR
+      // Use default user path
+      // In this case, userplugindir is a relative path to a prefix wich needs
+      // to be expanded
+      userplugindir = CS_DEFAULT_USER_PLUGINDIR;
+
+#if defined(LINUX) || defined(__MACH__)
+      char *prefix = getenv("HOME");
+#elif defined(WIN32)
+      char *prefix = getenv("LOCALAPPDATA");
+#endif
+      size_t prefixlen = strlen(prefix);
+      size_t userplugindirlen = strlen(userplugindir);
+      if(pos + prefixlen + 2 > searchpath_buflen - 1) {
+        csound->ErrorMsg(csound, Str("Plugins search path too long\n"));
+      } else if(userplugindirlen + prefixlen + 1 >= buflen) {
+        csound->ErrorMsg(csound, Str("User plugin dir too long\n"));
+      } else {
+        snprintf(buf, buflen, "%s/%s", prefix, userplugindir);
+        if(_dir_exists(buf)) {
+          snprintf(searchpath_buf, searchpath_buflen, "%s%c%s", dname, sep, buf);
+          dname = searchpath_buf;
+        }
+      }
+#endif
+    }
+
+    if(UNLIKELY(csound->oparms->odebug))
+      csound->Message(csound, Str("Plugins search path: %s\n"), dname);
+
+    /* We now loop through the directory list */
+    while(read_directory) {
+      /* find separator */
+    if((end = strchr(dname, sep)) != NULL) {
+      *end = '\0';
+      /* copy directory name */
+      dname1 = cs_strdup(csound, (char *) dname);
+
+      *end = sep;  /* restore for re-execution */
+      /* move to next directory name */
+      dname = end + 1;
+
+    } else {
+      /* copy last directory name) */
+      dname1 = cs_strdup(csound, (char *) dname);
+      read_directory = 0;
+    }
+
+    /* protect for the case where there is an
+       extra separator at the end */
+    if(*dname1 == '\0') {
+      csound->Free(csound, dname1);
+      break;
+    }
+    dir = opendir(dname1);
+    if (UNLIKELY(dir == (DIR*) NULL)) {
+ #if defined(__HAIKU__)
+        if(!dfltdir)
+ #endif
+      csound->Warning(csound, Str("Error opening plugin directory '%s': %s"),
+                               dname1, strerror(errno));
+      csound->Free(csound, dname1);
+      continue;
+    }
+    if(UNLIKELY(csound->oparms->odebug))
+      csound->Message(csound, "Opening plugin directory: %s\n", dname1);
+    /* load database for deferred plugin loading */
+/*     n = csoundLoadOpcodeDB(csound, dname); */
+/*     if (n != 0) */
+/*       return n; */
+    /* scan all files in directory */
+    while ((f = readdir(dir)) != NULL) {
+      fname = &(f->d_name[0]);
+      if (UNLIKELY(fname[0]=='_')) continue;
+      n = len = (int) strlen(fname);
+      if (UNLIKELY(fname[0]=='_')) continue;
+#if defined(WIN32)
+      strcpy(buf, "dll");
+      n -= 4;
+#elif defined(__MACH__)
+      strcpy(buf, "dylib");
+      n -= 6;
+#else
+      strcpy(buf, "so");
+      n -= 3;
+#endif
+      if (n <= 0 || fname[n] != '.')
+        continue;
+      i = 0;
+      do {
+        if (UNLIKELY((fname[++n] | (char) 0x20) != buf[i]))
+          break;
+      } while (buf[++i] != '\0');
+      if (buf[i] != '\0')
+        continue;
+      /* found a dynamic library, attempt to open it */
+      if (UNLIKELY(((int) strlen(dname) + len + 2) > 1024)) {
+        csound->Warning(csound, Str("path name too long, skipping '%s'"),
+                                fname);
+        continue;
+      }
+      /* printf("DEBUG %s(%d): possibly deny %s\n", __FILE__, __LINE__,fname); */
+      if (UNLIKELY(csoundCheckOpcodeDeny(csound, fname))) {
+        csoundWarning(csound, Str("Library %s omitted\n"), fname);
+        continue;
+      }
+
+      snprintf(buf, 1024, "%s%c%s", dname1, DIRSEP, fname);
+
+      if (UNLIKELY(csound->oparms->odebug)) {
+        csoundMessage(csound, Str("Loading '%s'\n"), buf);
+       }
+      n = csoundLoadExternal(csound, buf);
+      if (UNLIKELY(UNLIKELY(n == CSOUND_ERROR)))
+        continue;               /* ignore non-plugin files */
+      if (UNLIKELY(n < err))
+        err = n;                /* record serious errors */
+    }
+    closedir(dir);
+    csound->Free(csound, dname1);
+    }
+    return (err == CSOUND_INITIALIZATION ? CSOUND_ERROR : err);
+#else
+    return CSOUND_SUCCESS;
+#endif  /* HAVE_DIRENT_H */
+}
+
+
+static int cmp_func(const void *p1, const void *p2)
+{
+    return (strcmp(*((const char**) p1), *((const char**) p2)));
+}
+
+int csoundLoadExternals(CSOUND *csound)
+{
+    char    *s, **lst;
+    int     i, cnt, err;
+
+    s = csound->dl_opcodes_oplibs;
+    if (UNLIKELY(s == NULL || s[0] == '\0'))
+      return 0;
+    /* IV - Feb 19 2005 */
+    csound->dl_opcodes_oplibs = NULL;
+    csoundMessage(csound, Str("Loading command-line libraries:\n"));
+    cnt = 1;
+    i = 0;
+    do {
+      if (s[i] == ',')
+        cnt++;
+    } while (s[++i] != '\0');
+    lst = (char**) csound->Malloc(csound, sizeof(char*) * cnt);
+    i = cnt = 0;
+    lst[cnt++] = s;
+    do {
+      if (s[i] == ',') {
+        lst[cnt++] = &(s[i + 1]);
+        s[i] = '\0';
+      }
+    } while (s[++i] != '\0');
+    qsort((void*) lst, (size_t) cnt, sizeof(char*), cmp_func);
+    i = 0;
+    do {
+      char  *fname = lst[i];
+      if (fname[0] != '\0' && !(i && strcmp(fname, lst[i - 1]) == 0)) {
+        err = csoundLoadExternal(csound, fname);
+        if (UNLIKELY(err == CSOUND_INITIALIZATION || err == CSOUND_MEMORY))
+          csoundDie(csound, Str(" *** error loading '%s'"), fname);
+        else if (!err)
+          csoundMessage(csound, "  %s\n", fname);
+      }
+    } while (++i < cnt);
+    /* file list is no longer needed */
+    csound->Free(csound, lst);
+    csound->Free(csound, s);
+    return 0;
+}
+
+/**
+ * Call initialisation functions of all loaded modules that have a
+ * csoundModuleInit symbol, for Csound instance 'csound'.
+ * Return value is CSOUND_SUCCESS if there was no error, and CSOUND_ERROR if
+ * some modules could not be initialised.
+ */
+
+int csoundInitModules(CSOUND *csound)
+{
+    csoundModule_t  *m;
+    int             i, retval = CSOUND_SUCCESS;
+    /* For regular Csound, init_static_modules is not compiled or called.
+     * For some builds of Csound, e.g. for PNaCl, init_static_modules is
+     * compiled and called to initialize statically linked opcodes and other
+     * plugins that are dynamically loaded on other platforms.
+     */
+#if defined(INIT_STATIC_MODULES)
+    retval = init_static_modules(csound);
+#endif
+    /* call init functions */
+    for (m = (csoundModule_t*) csound->csmodule_db; m != NULL; m = m->nxt) {
+      i = csoundInitModule(csound, m);
+      if (UNLIKELY(i != CSOUND_SUCCESS && i < retval))
+        retval = i;
+    }
+    /* return with error code */
+    return retval;
+}
+
+/* load a plugin library and also initialise it */
+/* called on deferred loading of opcode plugins */
+
+int csoundLoadAndInitModule(CSOUND *csound, const char *fname)
+{
+    volatile jmp_buf  tmpExitJmp;
+    volatile int      err;
+
+    err = csoundLoadExternal(csound, fname);
+    if (UNLIKELY(err != 0))
+      return err;
+    memcpy((void*) &tmpExitJmp, (void*) &csound->exitjmp, sizeof(jmp_buf));
+    if (UNLIKELY((err = setjmp(csound->exitjmp)) != 0)) {
+      memcpy((void*) &csound->exitjmp, (void*) &tmpExitJmp, sizeof(jmp_buf));
+      return (err == (CSOUND_EXITJMP_SUCCESS + CSOUND_MEMORY) ?
+              CSOUND_MEMORY : CSOUND_INITIALIZATION);
+    }
+    /* NOTE: this depends on csound->csmodule_db being the most recently */
+    /* loaded plugin library */
+
+    err = csoundInitModule(csound, (csoundModule_t*) csound->csmodule_db);
+    memcpy((void*) &csound->exitjmp, (void*) &tmpExitJmp, sizeof(jmp_buf));
+
+    return err;
+}
+
+int csoundLoadAndInitModules(CSOUND *csound, const char *opdir)
+{
+#if (defined(HAVE_DIRENT_H) && (TARGET_OS_IPHONE == 0))
+    DIR             *dir;
+    struct dirent   *f;
+    const char      *dname, *fname;
+    char            buf[1024];
+    int             i, n, len, err = CSOUND_SUCCESS;
+    char            *dname1, *end;
+    int             read_directory = 1;
+    char sep =
+#ifdef WIN32
+    ';';
+#else
+    ':';
+#endif
+#ifdef __HAIKU__
+        int dfltdir = 0;
+#endif
+        // VL: check is not wanted
+        //if (UNLIKELY(csound->csmodule_db != NULL))
+        ///return CSOUND_ERROR;
+
+    /* open plugin directory */
+    // EM'2021: This seems to be dead code since opdir will never be NULL and
+    // the value of dname will be discarded, see "dname = opdir" later
+    dname = csoundGetEnv(csound, (sizeof(MYFLT) == sizeof(float) ?
+                                  plugindir_envvar : plugindir64_envvar));
+    if (dname == NULL) {
+#if ENABLE_OPCODEDIR_WARNINGS
+      csound->opcodedirWasOK = 0;
+#  ifdef USE_DOUBLE
+      dname = csoundGetEnv(csound, plugindir_envvar);
+      if (dname == NULL)
+#  endif
+#endif
 #ifdef  CS_DEFAULT_PLUGINDIR
         dname = CS_DEFAULT_PLUGINDIR;
  #ifdef __HAIKU__
@@ -452,6 +898,10 @@ int csoundLoadModules(CSOUND *csound)
 #endif
     }
 
+    if (opdir != NULL) {
+      dname = opdir;
+    }
+
     /* We now loop through the directory list */
     while(read_directory) {
       /* find separator */
@@ -532,7 +982,7 @@ int csoundLoadModules(CSOUND *csound)
       if (UNLIKELY(csound->oparms->odebug)) {
         csoundMessage(csound, Str("Loading '%s'\n"), buf);
        }
-      n = csoundLoadExternal(csound, buf);
+      n = csoundLoadAndInitModule(csound, buf);
       if (UNLIKELY(UNLIKELY(n == CSOUND_ERROR)))
         continue;               /* ignore non-plugin files */
       if (UNLIKELY(n < err))
@@ -547,153 +997,6 @@ int csoundLoadModules(CSOUND *csound)
 #endif  /* HAVE_DIRENT_H */
 }
 
-static int cmp_func(const void *p1, const void *p2)
-{
-    return (strcmp(*((const char**) p1), *((const char**) p2)));
-}
-
-int csoundLoadExternals(CSOUND *csound)
-{
-    char    *s, **lst;
-    int     i, cnt, err;
-
-    s = csound->dl_opcodes_oplibs;
-    if (UNLIKELY(s == NULL || s[0] == '\0'))
-      return 0;
-    /* IV - Feb 19 2005 */
-    csound->dl_opcodes_oplibs = NULL;
-    csoundMessage(csound, Str("Loading command-line libraries:\n"));
-    cnt = 1;
-    i = 0;
-    do {
-      if (s[i] == ',')
-        cnt++;
-    } while (s[++i] != '\0');
-    lst = (char**) csound->Malloc(csound, sizeof(char*) * cnt);
-    i = cnt = 0;
-    lst[cnt++] = s;
-    do {
-      if (s[i] == ',') {
-        lst[cnt++] = &(s[i + 1]);
-        s[i] = '\0';
-      }
-    } while (s[++i] != '\0');
-    qsort((void*) lst, (size_t) cnt, sizeof(char*), cmp_func);
-    i = 0;
-    do {
-      char  *fname = lst[i];
-      if (fname[0] != '\0' && !(i && strcmp(fname, lst[i - 1]) == 0)) {
-        err = csoundLoadExternal(csound, fname);
-        if (UNLIKELY(err == CSOUND_INITIALIZATION || err == CSOUND_MEMORY))
-          csoundDie(csound, Str(" *** error loading '%s'"), fname);
-        else if (!err)
-          csoundMessage(csound, "  %s\n", fname);
-      }
-    } while (++i < cnt);
-    /* file list is no longer needed */
-    csound->Free(csound, lst);
-    csound->Free(csound, s);
-    return 0;
-}
-
-/**
- * Initialise a single module.
- * Return value is CSOUND_SUCCESS if there was no error.
- */
-static CS_NOINLINE int csoundInitModule(CSOUND *csound, csoundModule_t *m)
-{
-    int     i;
-
-    if (m->PreInitFunc != NULL) {
-      if (m->fn.p.InitFunc != NULL) {
-        i = m->fn.p.InitFunc(csound);
-        if (UNLIKELY(i != 0)) {
-          print_module_error(csound, Str("Error starting module '%s'"),
-                                     &(m->name[0]), m, i);
-          return CSOUND_ERROR;
-        }
-      }
-    }
-    else {
-      /* deal with fgens if there are any */
-      if (m->fn.o.fgen_init != NULL) {
-        NGFENS  *names = m->fn.o.fgen_init(csound);
-        for (i = 0; names[i].name != NULL; i++)
-          allocgen(csound, names[i].name, names[i].fn);
-      }
-      if (m->fn.o.opcode_init != NULL) {
-        OENTRY  *opcodlst_n;
-        long    length;
-        /* load opcodes */
-        if (UNLIKELY((length = m->fn.o.opcode_init(csound, &opcodlst_n)) < 0L))
-          return CSOUND_ERROR;
-        else {
-          length /= (long) sizeof(OENTRY);
-          if (length) {
-            if (UNLIKELY(csoundAppendOpcodes(csound, opcodlst_n,
-                                               (int) length) != 0))
-              return CSOUND_ERROR;
-          }
-        }
-      }
-    }
-    return CSOUND_SUCCESS;
-}
-
-/**
- * Call initialisation functions of all loaded modules that have a
- * csoundModuleInit symbol, for Csound instance 'csound'.
- * Return value is CSOUND_SUCCESS if there was no error, and CSOUND_ERROR if
- * some modules could not be initialised.
- */
-
-int csoundInitModules(CSOUND *csound)
-{
-    csoundModule_t  *m;
-    int             i, retval = CSOUND_SUCCESS;
-    /* For regular Csound, init_static_modules is not compiled or called.
-     * For some builds of Csound, e.g. for PNaCl, init_static_modules is
-     * compiled and called to initialize statically linked opcodes and other
-     * plugins that are dynamically loaded on other platforms.
-     */
-#if defined(INIT_STATIC_MODULES)
-    retval = init_static_modules(csound);
-#endif
-    /* call init functions */
-    for (m = (csoundModule_t*) csound->csmodule_db; m != NULL; m = m->nxt) {
-      i = csoundInitModule(csound, m);
-      if (UNLIKELY(i != CSOUND_SUCCESS && i < retval))
-        retval = i;
-    }
-    /* return with error code */
-    return retval;
-}
-
-/* load a plugin library and also initialise it */
-/* called on deferred loading of opcode plugins */
-
-int csoundLoadAndInitModule(CSOUND *csound, const char *fname)
-{
-    volatile jmp_buf  tmpExitJmp;
-    volatile int      err;
-
-    err = csoundLoadExternal(csound, fname);
-    if (UNLIKELY(err != 0))
-      return err;
-    memcpy((void*) &tmpExitJmp, (void*) &csound->exitjmp, sizeof(jmp_buf));
-    if (UNLIKELY((err = setjmp(csound->exitjmp)) != 0)) {
-      memcpy((void*) &csound->exitjmp, (void*) &tmpExitJmp, sizeof(jmp_buf));
-      return (err == (CSOUND_EXITJMP_SUCCESS + CSOUND_MEMORY) ?
-              CSOUND_MEMORY : CSOUND_INITIALIZATION);
-    }
-    /* NOTE: this depends on csound->csmodule_db being the most recently */
-    /* loaded plugin library */
-    err = csoundInitModule(csound, (csoundModule_t*) csound->csmodule_db);
-    memcpy((void*) &csound->exitjmp, (void*) &tmpExitJmp, sizeof(jmp_buf));
-
-    return err;
-}
-
 /**
  * Call destructor functions of all loaded modules that have a
  * csoundModuleDestroy symbol, for Csound instance 'csound'.
@@ -731,6 +1034,8 @@ int csoundDestroyModules(CSOUND *csound)
     return retval;
 }
 
+#endif /* __wasi__ */
+
  /* ------------------------------------------------------------------------ */
 
 #if defined(WIN32)
@@ -751,7 +1056,7 @@ PUBLIC void *csoundGetLibrarySymbol(void *library, const char *procedureName)
     return (void*) GetProcAddress((HMODULE) library, procedureName);
 }
 
-#elif !(defined(NACL)) && (defined(LINUX) || defined(NEW_MACH_CODE) || defined(__HAIKU__))
+#elif !(defined(NACL)) && !(defined(__wasi__)) && (defined(LINUX) || defined(NEW_MACH_CODE) || defined(__HAIKU__))
 
 PUBLIC int csoundOpenLibrary(void **library, const char *libraryPath)
 {
@@ -827,65 +1132,93 @@ void print_opcodedir_warning(CSOUND *p)
 #endif
 }
 
-typedef long (*INITFN)(CSOUND *, void *);
+/**
+ Builtin linkage for C opcodes - Instructions:
+ - use csoundCore.h instead of csdl.h.
+ - name the OENTRY array <name>, where <name> is any arbitrary name.
+ - add the macro LINKAGE_BUILTIN(<name>) to the end of the file.
+ - append the init function prototype below
+   extern int64_t <name>_init(CSOUND *, void *);
+ - append the init function name <name>_init to the
+   staticmodules[] array initialisation.
+ - insert source code to libcsound_SRCS in../CMakeLists.txt
+*/
 
-extern long babo_localops_init(CSOUND *, void *);
-extern long bilbar_localops_init(CSOUND *, void *);
-extern long compress_localops_init(CSOUND *, void *);
-extern long pvsbuffer_localops_init(CSOUND *, void *);
-extern long vosim_localops_init(CSOUND *, void *);
-extern long eqfil_localops_init(CSOUND *, void *);
-extern long modal4_localops_init(CSOUND *, void *);
-extern long scoreline_localops_init(CSOUND *, void *);
-extern long physmod_localops_init(CSOUND *, void *);
-extern long modmatrix_localops_init(CSOUND *, void *);
-extern long spectra_localops_init(CSOUND *, void *);
-extern long ambicode1_localops_init(CSOUND *, void *);
-extern long grain4_localops_init(CSOUND *, void *);
-extern long hrtferX_localops_init(CSOUND *, void *);
-extern long loscilx_localops_init(CSOUND *, void *);
-extern long pan2_localops_init(CSOUND *, void *);
-extern long arrayvars_localops_init(CSOUND *, void *);
-extern long phisem_localops_init(CSOUND *, void *);
-extern long pvoc_localops_init(CSOUND *, void *);
-extern long hrtfopcodes_localops_init(CSOUND *, void *);
-extern long hrtfreverb_localops_init(CSOUND *, void *);
-extern long hrtfearly_localops_init(CSOUND *, void *);
-extern long minmax_localops_init(CSOUND *, void *);
-extern long gendy_localops_init(CSOUND *, void *);
-//extern long stackops_localops_init(CSOUND *, void *);
-extern long vbap_localops_init(CSOUND *, void *);
-extern long vaops_localops_init(CSOUND *, void*);
-extern long ugakbari_localops_init(CSOUND *, void *);
-extern long harmon_localops_init(CSOUND *, void *);
-extern long pitchtrack_localops_init(CSOUND *, void *);
-extern long squinewave_localops_init(CSOUND *, void *);
+typedef int32_t (*INITFN)(CSOUND *, void *);
 
-extern long partikkel_localops_init(CSOUND *, void *);
-extern long shape_localops_init(CSOUND *, void *);
-extern long tabaudio_localops_init(CSOUND *, void *);
-extern long tabsum_localops_init(CSOUND *, void *);
-extern long crossfm_localops_init(CSOUND *, void *);
-extern long pvlock_localops_init(CSOUND *, void *);
-extern long fareyseq_localops_init(CSOUND *, void *);
-extern long cpumeter_localops_init(CSOUND *, void *);
-extern long scnoise_localops_init(CSOUND *, void *);
-#ifndef NACL
-extern long socksend_localops_init(CSOUND *, void *);
-extern long mp3in_localops_init(CSOUND *, void *);
-extern long sockrecv_localops_init(CSOUND *, void *);
-#endif
-extern long afilts_localops_init(CSOUND *, void *);
-extern long pinker_localops_init(CSOUND *, void *);
-extern long paulstretch_localops_init(CSOUND *, void *);
-extern long wpfilters_localops_init(CSOUND *, void *);
-extern long zak_localops_init(CSOUND *, void *);
+extern int32_t babo_localops_init(CSOUND *, void *);
+extern int32_t bilbar_localops_init(CSOUND *, void *);
+extern int32_t compress_localops_init(CSOUND *, void *);
+extern int32_t pvsbuffer_localops_init(CSOUND *, void *);
+extern int32_t vosim_localops_init(CSOUND *, void *);
+extern int32_t eqfil_localops_init(CSOUND *, void *);
+extern int32_t modal4_localops_init(CSOUND *, void *);
+extern int32_t scoreline_localops_init(CSOUND *, void *);
+extern int32_t physmod_localops_init(CSOUND *, void *);
+extern int32_t modmatrix_localops_init(CSOUND *, void *);
+extern int32_t spectra_localops_init(CSOUND *, void *);
+extern int32_t ambicode1_localops_init(CSOUND *, void *);
+extern int32_t grain4_localops_init(CSOUND *, void *);
+extern int32_t hrtferX_localops_init(CSOUND *, void *);
+extern int32_t loscilx_localops_init(CSOUND *, void *);
+extern int32_t pan2_localops_init(CSOUND *, void *);
+extern int32_t arrayvars_localops_init(CSOUND *, void *);
+extern int32_t phisem_localops_init(CSOUND *, void *);
+extern int32_t pvoc_localops_init(CSOUND *, void *);
+extern int32_t hrtfopcodes_localops_init(CSOUND *, void *);
+extern int32_t hrtfreverb_localops_init(CSOUND *, void *);
+extern int32_t hrtfearly_localops_init(CSOUND *, void *);
+extern int32_t minmax_localops_init(CSOUND *, void *);
+extern int32_t gendy_localops_init(CSOUND *, void *);
+extern int32_t vbap_localops_init(CSOUND *, void *);
+extern int32_t vaops_localops_init(CSOUND *, void*);
+extern int32_t ugakbari_localops_init(CSOUND *, void *);
+extern int32_t harmon_localops_init(CSOUND *, void *);
+extern int32_t pitchtrack_localops_init(CSOUND *, void *);
+extern int32_t squinewave_localops_init(CSOUND *, void *);
 
-extern int stdopc_ModuleInit(CSOUND *csound);
-extern int pvsopc_ModuleInit(CSOUND *csound);
-extern int sfont_ModuleInit(CSOUND *csound);
-extern int sfont_ModuleCreate(CSOUND *csound);
-extern int newgabopc_ModuleInit(CSOUND *csound);
+extern int32_t partikkel_localops_init(CSOUND *, void *);
+extern int32_t shape_localops_init(CSOUND *, void *);
+extern int32_t tabaudio_localops_init(CSOUND *, void *);
+extern int32_t tabsum_localops_init(CSOUND *, void *);
+extern int32_t crossfm_localops_init(CSOUND *, void *);
+extern int32_t pvlock_localops_init(CSOUND *, void *);
+extern int32_t fareyseq_localops_init(CSOUND *, void *);
+extern int32_t cpumeter_localops_init(CSOUND *, void *);
+extern int32_t scnoise_localops_init(CSOUND *, void *);
+extern int32_t socksend_localops_init(CSOUND *, void *);
+extern int32_t mp3in_localops_init(CSOUND *, void *);
+extern int32_t sockrecv_localops_init(CSOUND *, void *);
+extern int32_t afilts_localops_init(CSOUND *, void *);
+extern int32_t pinker_localops_init(CSOUND *, void *);
+extern int32_t paulstretch_localops_init(CSOUND *, void *);
+extern int32_t wpfilters_localops_init(CSOUND *, void *);
+extern int32_t zak_localops_init(CSOUND *, void *);
+extern int32_t lufs_localops_init(CSOUND *, void *);
+extern int32_t sterrain_localops_init(CSOUND *, void *);
+extern int32_t date_localops_init(CSOUND *, void *);
+extern int32_t system_localops_init(CSOUND *, void *);
+extern int32_t liveconv_localops_init(CSOUND *, void *);
+extern int32_t gamma_localops_init(CSOUND *, void *);
+extern int32_t quadbezier_localops_init(CSOUND *, void *);
+extern int32_t framebuffer_localops_init(CSOUND *, void *);
+extern int32_t cell_localops_init(CSOUND *, void *);
+extern int32_t exciter_localops_init(CSOUND *, void *);
+extern int32_t buchla_localops_init(CSOUND *, void *);
+extern int32_t select_localops_init(CSOUND *, void *);
+extern int32_t serial_localops_init(CSOUND *, void *);
+extern int32_t counter_localops_init(CSOUND *, void *);
+extern int32_t platerev_localops_init(CSOUND *, void *);
+extern int32_t sequencer_localops_init(CSOUND *, void *);
+extern int32_t pvsgendy_localops_init(CSOUND *, void *);
+extern int32_t scugens_localops_init(CSOUND *, void *);
+extern int32_t emugens_localops_init(CSOUND *, void *);
+
+extern int32_t stdopc_ModuleInit(CSOUND *csound);
+extern int32_t pvsopc_ModuleInit(CSOUND *csound);
+extern int32_t sfont_ModuleInit(CSOUND *csound);
+extern int32_t sfont_ModuleCreate(CSOUND *csound);
+extern int32_t newgabopc_ModuleInit(CSOUND *csound);
 
 const INITFN staticmodules[] = { hrtfopcodes_localops_init, babo_localops_init,
                                  bilbar_localops_init, vosim_localops_init,
@@ -909,36 +1242,60 @@ const INITFN staticmodules[] = { hrtfopcodes_localops_init, babo_localops_init,
 #ifdef LINUX
                                  cpumeter_localops_init,
 #endif
-#ifndef NACL
-                                 mp3in_localops_init,
+
+#if !(defined(NACL)) && !(defined(__wasi__))
+                                 counter_localops_init,
                                  sockrecv_localops_init,
                                  socksend_localops_init,
+                                 system_localops_init,
+                                 serial_localops_init,
+                                 mp3in_localops_init,
 #endif
                                  scnoise_localops_init, afilts_localops_init,
                                  pinker_localops_init, gendy_localops_init,
                                  wpfilters_localops_init, zak_localops_init,
+                                 lufs_localops_init, sterrain_localops_init,
+                                 date_localops_init,
+                                 liveconv_localops_init, gamma_localops_init,
+                                 framebuffer_localops_init, cell_localops_init,
+                                 exciter_localops_init, buchla_localops_init,
+                                 select_localops_init,
+                                 platerev_localops_init,
+                                 pvsgendy_localops_init, scugens_localops_init,
+                                 emugens_localops_init, sequencer_localops_init,
                                  NULL };
 
+/**
+ Builtin linkage for C fgens - Instructions:
+ - use csoundCore.h instead of csdl.h.
+ - name the NGFENS array <name>, where <name> is any arbitrary name.
+ - add the macro FLINKAGE_BUILTIN(<name>) to the end of the file.
+ - append the init function prototype below
+   NGFENS* <name>_init(CSOUND *);
+ - append the init function name <name>_init to the
+   ftgenab[] array initialisation.
+ - insert source code to libcsound_SRCS in../CMakeLists.txt
+*/
 typedef NGFENS* (*FGINITFN)(CSOUND *);
 
 NGFENS *ftest_fgens_init(CSOUND *);
+NGFENS *farey_fgens_init(CSOUND *);
 
-const FGINITFN fgentab[] = {  ftest_fgens_init, NULL };
+const FGINITFN fgentab[] = {  ftest_fgens_init, farey_fgens_init, NULL };
 
 CS_NOINLINE int csoundInitStaticModules(CSOUND *csound)
 {
     int     i;
     OENTRY  *opcodlst_n;
-    long    length;
+    int32_t length;
 
     for (i=0; staticmodules[i]!=NULL; i++) {
       length = (staticmodules[i])(csound, &opcodlst_n);
 
       if (UNLIKELY(length <= 0L)) return CSOUND_ERROR;
-      length /= (long) sizeof(OENTRY);
+      length /= (int32_t) sizeof(OENTRY);
       if (length) {
-        if (UNLIKELY(csoundAppendOpcodes(csound, opcodlst_n,
-                                           (int) length) != 0))
+        if (UNLIKELY(csoundAppendOpcodes(csound, opcodlst_n, length) != 0))
           return CSOUND_ERROR;
       }
     }
