@@ -196,12 +196,17 @@ extern int ISSTRCOD(MYFLT);
 #define INF     (2147483647.0)
 #define ROOT2   (1.414213562373095048801688724209698078569)
 
-#define AMPLMSG 01
-#define RNGEMSG 02
-#define WARNMSG 04
-#define RAWMSG  0x40
-#define TIMEMSG 0x80
-#define NOQQ    0x400
+/* CONSTANTS FOR USE IN MSGLEVEL */
+#define CS_AMPLMSG 01
+#define CS_RNGEMSG 02
+#define CS_WARNMSG 04
+  //#define CS_UNUSED1 08
+#define CS_NOMSG   0x10
+  //#define CS_UNUSED2 0x20
+#define CS_RAWMSG  0x40
+#define CS_TIMEMSG 0x80
+#define CS_NOQQ    0x400
+
 #define IGN(X)  (void) X
 
 #define ARG_CONSTANT 0
@@ -263,6 +268,8 @@ typedef struct CORFIL {
     int     ksmps_override;
     int     fft_lib;
     int     echo;
+    MYFLT   limiter;
+    float   sr_default, kr_default;
   } OPARMS;
 
   typedef struct arglst {
@@ -1395,11 +1402,19 @@ typedef struct _message_queue_t_ {
     int (*GetErrorCnt)(CSOUND *);
     FUNC* (*FTnp2Finde)(CSOUND*, MYFLT *);
     INSTRTXT *(*GetInstrument)(CSOUND*, int, const char *);
+    MYFLT* (*AutoCorrelation)(CSOUND *, MYFLT*, MYFLT*, int, MYFLT*, int);
+    void * (*LPsetup)(CSOUND *csound, int N, int M);
+    void (*LPfree)(CSOUND *csound, void *);
+    MYFLT* (*LPred)(CSOUND *, void *, MYFLT *);
+    MYFLT* (*LPCeps)(CSOUND *, MYFLT *, MYFLT *, int, int);
+    MYFLT* (*CepsLP)(CSOUND *, MYFLT *, MYFLT *, int, int);
+    MYFLT (*LPrms)(CSOUND *, void *);
+    void *(*CreateThread2)(uintptr_t (*threadRoutine)(void *), unsigned int, void *userdata);
     /**@}*/
     /** @name Placeholders
         To allow the API to grow while maintining backward binary compatibility. */
     /**@{ */
-    SUBR dummyfn_2[29];
+    SUBR dummyfn_2[22];
     /**@}*/
 #ifdef __BUILDING_LIBCSOUND
     /* ------- private data (not to be used by hosts or externals) ------- */
@@ -1782,7 +1797,7 @@ typedef struct _message_queue_t_ {
     int           strsiz;       /* length of current strings space */
     FUNC          *sinetable;   /* A useful table */
     int           sinelength;   /* Size of table */
-    MYFLT         *powerof2;    /* pow2 table */
+    MYFLT         *UNUSEDP;     /* pow2 table */
     MYFLT         *cpsocfrc;    /* cps conv table */
     CORFIL*       expanded_orc; /* output of preprocessor */
     CORFIL*       expanded_sco; /* output of preprocessor */
@@ -1797,7 +1812,7 @@ typedef struct _message_queue_t_ {
                                and nodebug function */
     int           score_parser;
     CS_HASH_TABLE* symbtab;
-    int           unused_int1;
+    int           print_version;
     int           inZero;       /* flag compilation of instr0 */
     struct _message_queue **msg_queue;
     volatile long msg_queue_wget; /* Writer - Get index */
@@ -1819,6 +1834,10 @@ typedef struct _message_queue_t_ {
     unsigned long message_string_queue_wp;
     message_string_queue_t *message_string_queue;
     int io_initialised;
+    char *op;
+    int  mode;
+    char *opcodedir;
+    char *score_srt;
     /*struct CSOUND_ **self;*/
     /**@}*/
 #endif  /* __BUILDING_LIBCSOUND */
